"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol messages for describing the results of benchmarks and unit tests."""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class EntryValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOUBLE_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    double_value: builtins.float
    string_value: builtins.str
    def __init__(self, *, double_value: builtins.float | None = ..., string_value: builtins.str | None = ...) -> None: ...
    def HasField(
        self, field_name: typing.Literal["double_value", b"double_value", "kind", b"kind", "string_value", b"string_value"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["double_value", b"double_value", "kind", b"kind", "string_value", b"string_value"]
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["kind", b"kind"]
    ) -> typing.Literal["double_value", "string_value"] | None: ...

global___EntryValue = EntryValue

@typing.final
class MetricEntry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    MIN_VALUE_FIELD_NUMBER: builtins.int
    MAX_VALUE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Metric name"""
    value: builtins.float
    """Metric value"""
    @property
    def min_value(self) -> google.protobuf.wrappers_pb2.DoubleValue:
        """The minimum acceptable value for the metric if specified"""

    @property
    def max_value(self) -> google.protobuf.wrappers_pb2.DoubleValue:
        """The maximum acceptable value for the metric if specified"""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        value: builtins.float | None = ...,
        min_value: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        max_value: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["max_value", b"max_value", "min_value", b"min_value"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["max_value", b"max_value", "min_value", b"min_value", "name", b"name", "value", b"value"]
    ) -> None: ...

global___MetricEntry = MetricEntry

@typing.final
class BenchmarkEntry(google.protobuf.message.Message):
    """Each unit test or benchmark in a test or benchmark run provides
    some set of information.  Here we provide some reasonable keys
    one would expect to see, with optional key/value pairs for things
    we haven't considered.

    This BenchmarkEntry should be emitted by each unit test or benchmark
    reporter.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ExtrasEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___EntryValue: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___EntryValue | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    ITERS_FIELD_NUMBER: builtins.int
    CPU_TIME_FIELD_NUMBER: builtins.int
    WALL_TIME_FIELD_NUMBER: builtins.int
    THROUGHPUT_FIELD_NUMBER: builtins.int
    EXTRAS_FIELD_NUMBER: builtins.int
    METRICS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of the specific benchmark or test
    (e.g. BM_AdjustContrast_gpu_B_W_H)
    """
    iters: builtins.int
    """If a benchmark, how many iterations it was run for"""
    cpu_time: builtins.float
    """Total cpu time used for all iterations (in seconds)"""
    wall_time: builtins.float
    """Total wall time used for all iterations (in seconds)"""
    throughput: builtins.float
    """Throughput (in MB/s)"""
    @property
    def extras(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___EntryValue]:
        """Generic map from result key to value."""

    @property
    def metrics(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MetricEntry]:
        """Metric name, value and expected range. This can include accuracy metrics
        typically used to determine whether the accuracy test has passed
        """

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        iters: builtins.int | None = ...,
        cpu_time: builtins.float | None = ...,
        wall_time: builtins.float | None = ...,
        throughput: builtins.float | None = ...,
        extras: collections.abc.Mapping[builtins.str, global___EntryValue] | None = ...,
        metrics: collections.abc.Iterable[global___MetricEntry] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cpu_time",
            b"cpu_time",
            "extras",
            b"extras",
            "iters",
            b"iters",
            "metrics",
            b"metrics",
            "name",
            b"name",
            "throughput",
            b"throughput",
            "wall_time",
            b"wall_time",
        ],
    ) -> None: ...

global___BenchmarkEntry = BenchmarkEntry

@typing.final
class BenchmarkEntries(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTRY_FIELD_NUMBER: builtins.int
    @property
    def entry(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BenchmarkEntry]: ...
    def __init__(self, *, entry: collections.abc.Iterable[global___BenchmarkEntry] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["entry", b"entry"]) -> None: ...

global___BenchmarkEntries = BenchmarkEntries

@typing.final
class BuildConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODE_FIELD_NUMBER: builtins.int
    CC_FLAGS_FIELD_NUMBER: builtins.int
    OPTS_FIELD_NUMBER: builtins.int
    mode: builtins.str
    """opt, dbg, etc"""
    @property
    def cc_flags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """CC compiler flags, if known"""

    @property
    def opts(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Bazel compilation options, if known"""

    def __init__(
        self,
        *,
        mode: builtins.str | None = ...,
        cc_flags: collections.abc.Iterable[builtins.str] | None = ...,
        opts: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cc_flags", b"cc_flags", "mode", b"mode", "opts", b"opts"]) -> None: ...

global___BuildConfiguration = BuildConfiguration

@typing.final
class CommitId(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHANGELIST_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    SNAPSHOT_FIELD_NUMBER: builtins.int
    PENDING_CHANGELIST_FIELD_NUMBER: builtins.int
    changelist: builtins.int
    """Submitted changelist."""
    hash: builtins.str
    snapshot: builtins.str
    """Hash of intermediate change between hash/changelist and what was tested.
    Not used if the build is from a commit without modifications.
    """
    pending_changelist: builtins.int
    """Changelist tested if the change list is not already submitted."""
    def __init__(
        self,
        *,
        changelist: builtins.int | None = ...,
        hash: builtins.str | None = ...,
        snapshot: builtins.str | None = ...,
        pending_changelist: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["changelist", b"changelist", "hash", b"hash", "kind", b"kind"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "changelist",
            b"changelist",
            "hash",
            b"hash",
            "kind",
            b"kind",
            "pending_changelist",
            b"pending_changelist",
            "snapshot",
            b"snapshot",
        ],
    ) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["kind", b"kind"]) -> typing.Literal["changelist", "hash"] | None: ...

global___CommitId = CommitId

@typing.final
class CPUInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class CacheSizeEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.int
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.int | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NUM_CORES_FIELD_NUMBER: builtins.int
    NUM_CORES_ALLOWED_FIELD_NUMBER: builtins.int
    MHZ_PER_CPU_FIELD_NUMBER: builtins.int
    CPU_INFO_FIELD_NUMBER: builtins.int
    CPU_GOVERNOR_FIELD_NUMBER: builtins.int
    CACHE_SIZE_FIELD_NUMBER: builtins.int
    num_cores: builtins.int
    num_cores_allowed: builtins.int
    mhz_per_cpu: builtins.float
    """How fast are these cpus?"""
    cpu_info: builtins.str
    """Additional cpu information. For example,
    Intel Ivybridge with HyperThreading (24 cores) dL1:32KB dL2:256KB dL3:30MB
    """
    cpu_governor: builtins.str
    """What kind of cpu scaling is enabled on the host.
    Examples include "performance", "ondemand", "conservative", "mixed".
    """
    @property
    def cache_size(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.int]:
        """Cache sizes (in bytes), e.g. "L2": 262144 (for 256KB)"""

    def __init__(
        self,
        *,
        num_cores: builtins.int | None = ...,
        num_cores_allowed: builtins.int | None = ...,
        mhz_per_cpu: builtins.float | None = ...,
        cpu_info: builtins.str | None = ...,
        cpu_governor: builtins.str | None = ...,
        cache_size: collections.abc.Mapping[builtins.str, builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cache_size",
            b"cache_size",
            "cpu_governor",
            b"cpu_governor",
            "cpu_info",
            b"cpu_info",
            "mhz_per_cpu",
            b"mhz_per_cpu",
            "num_cores",
            b"num_cores",
            "num_cores_allowed",
            b"num_cores_allowed",
        ],
    ) -> None: ...

global___CPUInfo = CPUInfo

@typing.final
class MemoryInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOTAL_FIELD_NUMBER: builtins.int
    AVAILABLE_FIELD_NUMBER: builtins.int
    total: builtins.int
    """Total virtual memory in bytes"""
    available: builtins.int
    """Immediately available memory in bytes"""
    def __init__(self, *, total: builtins.int | None = ..., available: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["available", b"available", "total", b"total"]) -> None: ...

global___MemoryInfo = MemoryInfo

@typing.final
class GPUInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_FIELD_NUMBER: builtins.int
    UUID_FIELD_NUMBER: builtins.int
    BUS_ID_FIELD_NUMBER: builtins.int
    model: builtins.str
    """e.g. "Tesla K40c" """
    uuid: builtins.str
    """Final entry in output of "nvidia-smi -L" """
    bus_id: builtins.str
    """e.g. "0000:04:00.0" """
    def __init__(
        self, *, model: builtins.str | None = ..., uuid: builtins.str | None = ..., bus_id: builtins.str | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["bus_id", b"bus_id", "model", b"model", "uuid", b"uuid"]) -> None: ...

global___GPUInfo = GPUInfo

@typing.final
class PlatformInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BITS_FIELD_NUMBER: builtins.int
    LINKAGE_FIELD_NUMBER: builtins.int
    MACHINE_FIELD_NUMBER: builtins.int
    RELEASE_FIELD_NUMBER: builtins.int
    SYSTEM_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    bits: builtins.str
    """e.g. '64bit'"""
    linkage: builtins.str
    """e.g. 'ELF'"""
    machine: builtins.str
    """e.g. 'i386'"""
    release: builtins.str
    """e.g. '3.13.0-76-generic'"""
    system: builtins.str
    """e.g. 'Linux'"""
    version: builtins.str
    """e.g. '#120-Ubuntu SMP Mon Jan 18 15:59:10 UTC 2016'"""
    def __init__(
        self,
        *,
        bits: builtins.str | None = ...,
        linkage: builtins.str | None = ...,
        machine: builtins.str | None = ...,
        release: builtins.str | None = ...,
        system: builtins.str | None = ...,
        version: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bits",
            b"bits",
            "linkage",
            b"linkage",
            "machine",
            b"machine",
            "release",
            b"release",
            "system",
            b"system",
            "version",
            b"version",
        ],
    ) -> None: ...

global___PlatformInfo = PlatformInfo

@typing.final
class AvailableDeviceInfo(google.protobuf.message.Message):
    """Matches DeviceAttributes"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    MEMORY_LIMIT_FIELD_NUMBER: builtins.int
    PHYSICAL_DESCRIPTION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Device name."""
    type: builtins.str
    """Device type, e.g. 'CPU' or 'GPU'."""
    memory_limit: builtins.int
    """Memory capacity in bytes."""
    physical_description: builtins.str
    """The physical description of this device."""
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        type: builtins.str | None = ...,
        memory_limit: builtins.int | None = ...,
        physical_description: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "memory_limit", b"memory_limit", "name", b"name", "physical_description", b"physical_description", "type", b"type"
        ],
    ) -> None: ...

global___AvailableDeviceInfo = AvailableDeviceInfo

@typing.final
class MachineConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HOSTNAME_FIELD_NUMBER: builtins.int
    SERIAL_IDENTIFIER_FIELD_NUMBER: builtins.int
    PLATFORM_INFO_FIELD_NUMBER: builtins.int
    CPU_INFO_FIELD_NUMBER: builtins.int
    DEVICE_INFO_FIELD_NUMBER: builtins.int
    AVAILABLE_DEVICE_INFO_FIELD_NUMBER: builtins.int
    MEMORY_INFO_FIELD_NUMBER: builtins.int
    hostname: builtins.str
    """Host name of machine that ran the benchmark."""
    serial_identifier: builtins.str
    """Unique serial number of the machine."""
    @property
    def platform_info(self) -> global___PlatformInfo:
        """Additional platform information."""

    @property
    def cpu_info(self) -> global___CPUInfo:
        """CPU Information."""

    @property
    def device_info(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """Other devices that are attached and relevant (e.g. GPUInfo)."""

    @property
    def available_device_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AvailableDeviceInfo]:
        """Devices accessible to the test (e.g. as given by list_local_devices)."""

    @property
    def memory_info(self) -> global___MemoryInfo: ...
    def __init__(
        self,
        *,
        hostname: builtins.str | None = ...,
        serial_identifier: builtins.str | None = ...,
        platform_info: global___PlatformInfo | None = ...,
        cpu_info: global___CPUInfo | None = ...,
        device_info: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
        available_device_info: collections.abc.Iterable[global___AvailableDeviceInfo] | None = ...,
        memory_info: global___MemoryInfo | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal["cpu_info", b"cpu_info", "memory_info", b"memory_info", "platform_info", b"platform_info"],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "available_device_info",
            b"available_device_info",
            "cpu_info",
            b"cpu_info",
            "device_info",
            b"device_info",
            "hostname",
            b"hostname",
            "memory_info",
            b"memory_info",
            "platform_info",
            b"platform_info",
            "serial_identifier",
            b"serial_identifier",
        ],
    ) -> None: ...

global___MachineConfiguration = MachineConfiguration

@typing.final
class RunConfiguration(google.protobuf.message.Message):
    """Run-specific items such as arguments to the test / benchmark."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class EnvVarsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    ARGUMENT_FIELD_NUMBER: builtins.int
    ENV_VARS_FIELD_NUMBER: builtins.int
    @property
    def argument(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def env_vars(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Environment variables used to run the test/benchmark."""

    def __init__(
        self,
        *,
        argument: collections.abc.Iterable[builtins.str] | None = ...,
        env_vars: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["argument", b"argument", "env_vars", b"env_vars"]) -> None: ...

global___RunConfiguration = RunConfiguration

@typing.final
class TestResults(google.protobuf.message.Message):
    """The output of one benchmark / test run.  Each run contains a list of
    tests or benchmarks, stored as BenchmarkEntry messages.

    This message should be emitted by the reporter (which runs the
    test / BM in a subprocess and then reads the emitted BenchmarkEntry messages;
    usually from a serialized json file, finally collecting them along
    with additional information about the test run.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _BenchmarkType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _BenchmarkTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[TestResults._BenchmarkType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: TestResults._BenchmarkType.ValueType  # 0
        """Fallback for protos written before Type was introduced."""
        CPP_MICROBENCHMARK: TestResults._BenchmarkType.ValueType  # 1
        PYTHON_BENCHMARK: TestResults._BenchmarkType.ValueType  # 2
        ANDROID_BENCHMARK: TestResults._BenchmarkType.ValueType  # 3
        EDGE_BENCHMARK: TestResults._BenchmarkType.ValueType  # 4
        IOS_BENCHMARK: TestResults._BenchmarkType.ValueType  # 5

    class BenchmarkType(_BenchmarkType, metaclass=_BenchmarkTypeEnumTypeWrapper):
        """The type of benchmark."""

    UNKNOWN: TestResults.BenchmarkType.ValueType  # 0
    """Fallback for protos written before Type was introduced."""
    CPP_MICROBENCHMARK: TestResults.BenchmarkType.ValueType  # 1
    PYTHON_BENCHMARK: TestResults.BenchmarkType.ValueType  # 2
    ANDROID_BENCHMARK: TestResults.BenchmarkType.ValueType  # 3
    EDGE_BENCHMARK: TestResults.BenchmarkType.ValueType  # 4
    IOS_BENCHMARK: TestResults.BenchmarkType.ValueType  # 5

    TARGET_FIELD_NUMBER: builtins.int
    ENTRIES_FIELD_NUMBER: builtins.int
    BUILD_CONFIGURATION_FIELD_NUMBER: builtins.int
    COMMIT_ID_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    RUN_TIME_FIELD_NUMBER: builtins.int
    MACHINE_CONFIGURATION_FIELD_NUMBER: builtins.int
    RUN_CONFIGURATION_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    BENCHMARK_TYPE_FIELD_NUMBER: builtins.int
    RUN_MODE_FIELD_NUMBER: builtins.int
    TF_VERSION_FIELD_NUMBER: builtins.int
    target: builtins.str
    """The target of the run, e.g.:
     //tensorflow/core:kernels_adjust_contrast_op_benchmark_test
    """
    start_time: builtins.int
    """The time the run started (in seconds of UTC time since Unix epoch)"""
    run_time: builtins.float
    """The amount of time the total run took (wall time in seconds)"""
    name: builtins.str
    """Benchmark target identifier."""
    benchmark_type: global___TestResults.BenchmarkType.ValueType
    run_mode: builtins.str
    """Used for differentiating between continuous and debug builds.
    Must be one of:
    * cbuild: results from continuous build.
    * presubmit: results from oneshot requests.
    * culprit: results from culprit finder rerun.
    """
    tf_version: builtins.str
    """TensorFlow version this benchmark runs against.
    This can be either set to full version or just the major version.
    """
    @property
    def entries(self) -> global___BenchmarkEntries:
        """The list of tests or benchmarks in this run."""

    @property
    def build_configuration(self) -> global___BuildConfiguration:
        """The configuration of the build (compiled opt? with cuda? any copts?)"""

    @property
    def commit_id(self) -> global___CommitId:
        """The commit id (git hash or changelist)"""

    @property
    def machine_configuration(self) -> global___MachineConfiguration:
        """Machine-specific parameters (Platform and CPU info)"""

    @property
    def run_configuration(self) -> global___RunConfiguration:
        """Run-specific parameters (arguments, etc)"""

    def __init__(
        self,
        *,
        target: builtins.str | None = ...,
        entries: global___BenchmarkEntries | None = ...,
        build_configuration: global___BuildConfiguration | None = ...,
        commit_id: global___CommitId | None = ...,
        start_time: builtins.int | None = ...,
        run_time: builtins.float | None = ...,
        machine_configuration: global___MachineConfiguration | None = ...,
        run_configuration: global___RunConfiguration | None = ...,
        name: builtins.str | None = ...,
        benchmark_type: global___TestResults.BenchmarkType.ValueType | None = ...,
        run_mode: builtins.str | None = ...,
        tf_version: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "build_configuration",
            b"build_configuration",
            "commit_id",
            b"commit_id",
            "entries",
            b"entries",
            "machine_configuration",
            b"machine_configuration",
            "run_configuration",
            b"run_configuration",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "benchmark_type",
            b"benchmark_type",
            "build_configuration",
            b"build_configuration",
            "commit_id",
            b"commit_id",
            "entries",
            b"entries",
            "machine_configuration",
            b"machine_configuration",
            "name",
            b"name",
            "run_configuration",
            b"run_configuration",
            "run_mode",
            b"run_mode",
            "run_time",
            b"run_time",
            "start_time",
            b"start_time",
            "target",
            b"target",
            "tf_version",
            b"tf_version",
        ],
    ) -> None: ...

global___TestResults = TestResults



from _typeshed import Incomplete
from collections.abc import Generator
from typing import Literal, overload

def htmlentityreplace_errors(exc: UnicodeError) -> tuple[str | bytes, int]: ...
@overload
def serialize(
    input,
    tree: Literal["dom", "genshi", "lxml", "etree"] = "etree",
    encoding: Literal[""] | None = None,
    *,
    quote_attr_values: Literal["legacy", "spec", "always"] = "legacy",
    quote_char: str = '"',
    use_best_quote_char: bool = ...,  # default value depends on whether quote_char was passed
    omit_optional_tags: bool = True,
    minimize_boolean_attributes: bool = True,
    use_trailing_solidus: bool = False,
    space_before_trailing_solidus: bool = True,
    escape_lt_in_attrs: bool = False,
    escape_rcdata: bool = False,
    resolve_entities: bool = True,
    alphabetical_attributes: bool = False,
    inject_meta_charset: bool = True,
    strip_whitespace: bool = False,
    sanitize: bool = False,
) -> str: ...
@overload
def serialize(
    input,
    tree: Literal["dom", "genshi", "lxml", "etree"] = "etree",
    encoding: str = ...,
    *,
    quote_attr_values: Literal["legacy", "spec", "always"] = "legacy",
    quote_char: str = '"',
    use_best_quote_char: bool = ...,  # default value depends on whether quote_char was passed
    omit_optional_tags: bool = True,
    minimize_boolean_attributes: bool = True,
    use_trailing_solidus: bool = False,
    space_before_trailing_solidus: bool = True,
    escape_lt_in_attrs: bool = False,
    escape_rcdata: bool = False,
    resolve_entities: bool = True,
    alphabetical_attributes: bool = False,
    inject_meta_charset: bool = True,
    strip_whitespace: bool = False,
    sanitize: bool = False,
) -> bytes: ...

class HTMLSerializer:
    quote_attr_values: Literal["legacy", "spec", "always"]
    quote_char: str
    use_best_quote_char: bool
    omit_optional_tags: bool
    minimize_boolean_attributes: bool
    use_trailing_solidus: bool
    space_before_trailing_solidus: bool
    escape_lt_in_attrs: bool
    escape_rcdata: bool
    resolve_entities: bool
    alphabetical_attributes: bool
    inject_meta_charset: bool
    strip_whitespace: bool
    sanitize: bool
    options: tuple[str, ...]
    errors: list[Incomplete]
    strict: bool
    def __init__(
        self,
        *,
        quote_attr_values: Literal["legacy", "spec", "always"] = "legacy",
        quote_char: str = '"',
        use_best_quote_char: bool = ...,  # default value depends on whether quote_char was passed
        omit_optional_tags: bool = True,
        minimize_boolean_attributes: bool = True,
        use_trailing_solidus: bool = False,
        space_before_trailing_solidus: bool = True,
        escape_lt_in_attrs: bool = False,
        escape_rcdata: bool = False,
        resolve_entities: bool = True,
        alphabetical_attributes: bool = False,
        inject_meta_charset: bool = True,
        strip_whitespace: bool = False,
        sanitize: bool = False,
    ) -> None: ...
    def encode(self, string: str) -> str | bytes: ...  # result depends on self.encoding
    def encodeStrict(self, string: str) -> str | bytes: ...  # result depends on self.encoding
    encoding: str | None
    @overload
    def serialize(self, treewalker, encoding: Literal[""] | None = None) -> Generator[str]: ...
    @overload
    def serialize(self, treewalker, encoding: str = ...) -> Generator[bytes]: ...
    @overload
    def render(self, treewalker, encoding: Literal[""] | None = None) -> str: ...
    @overload
    def render(self, treewalker, encoding: str = ...) -> bytes: ...
    def serializeError(self, data="XXX ERROR MESSAGE NEEDED") -> None: ...

class SerializeError(Exception): ...



from _typeshed import Incomplete
from typing import Final, NoReturn

from reportlab.graphics.shapes import LineShape
from reportlab.graphics.widgetbase import Widget

__version__: Final[str]

def frange(start, end=None, inc=None): ...
def makeDistancesList(list): ...

class Grid(Widget):
    x: int
    y: int
    width: int
    height: int
    orientation: str
    useLines: int
    useRects: int
    delta: int
    delta0: int
    deltaSteps: Incomplete
    fillColor: Incomplete
    stripeColors: Incomplete
    strokeColor: Incomplete
    strokeWidth: int
    def __init__(self) -> None: ...
    def demo(self): ...
    def makeOuterRect(self): ...
    def makeLinePosList(self, start, isX: int = 0): ...
    def makeInnerLines(self): ...
    def makeInnerTiles(self): ...
    def draw(self): ...

class DoubleGrid(Widget):
    x: int
    y: int
    width: int
    height: int
    grid0: Incomplete
    grid1: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    def draw(self): ...

class ShadedRect(Widget):
    x: int
    y: int
    width: int
    height: int
    orientation: str
    numShades: int
    fillColorStart: Incomplete
    fillColorEnd: Incomplete
    strokeColor: Incomplete
    strokeWidth: int
    cylinderMode: int
    def __init__(self, **kw) -> None: ...
    def demo(self): ...
    def draw(self): ...

def colorRange(c0, c1, n): ...
def centroid(P): ...
def rotatedEnclosingRect(P, angle, rect): ...

class ShadedPolygon(Widget, LineShape):
    angle: int
    fillColorStart: Incomplete
    fillColorEnd: Incomplete
    cylinderMode: int
    numShades: int
    points: Incomplete
    def __init__(self, **kw) -> None: ...
    def draw(self): ...
    # NOTE: widgets don't implement this, only actual shapes
    def copy(self) -> NoReturn: ...



from typing_extensions import Buffer

def ige256_encrypt(data: Buffer, key: Buffer, iv: Buffer, /) -> bytes: ...
def ige256_decrypt(data: Buffer, key: Buffer, iv: Buffer, /) -> bytes: ...
def ctr256_encrypt(data: Buffer, key: Buffer, iv: Buffer, state: Buffer, /) -> bytes: ...
def ctr256_decrypt(data: Buffer, key: Buffer, iv: Buffer, state: Buffer, /) -> bytes: ...
def cbc256_encrypt(data: Buffer, key: Buffer, iv: Buffer, /) -> bytes: ...
def cbc256_decrypt(data: Buffer, key: Buffer, iv: Buffer, /) -> bytes: ...



import enum
import sys
import types
from _typeshed import DataclassInstance
from builtins import type as Type  # alias to avoid name clashes with fields named "type"
from collections.abc import Callable, Iterable, Mapping
from types import GenericAlias
from typing import Any, Final, Generic, Literal, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Never, TypeIs

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)

__all__ = [
    "dataclass",
    "field",
    "Field",
    "FrozenInstanceError",
    "InitVar",
    "MISSING",
    "fields",
    "asdict",
    "astuple",
    "make_dataclass",
    "replace",
    "is_dataclass",
]

if sys.version_info >= (3, 10):
    __all__ += ["KW_ONLY"]

_DataclassT = TypeVar("_DataclassT", bound=DataclassInstance)

@type_check_only
class _DataclassFactory(Protocol):
    def __call__(
        self,
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type[_T]: ...

# define _MISSING_TYPE as an enum within the type stubs,
# even though that is not really its type at runtime
# this allows us to use Literal[_MISSING_TYPE.MISSING]
# for background, see:
#   https://github.com/python/typeshed/pull/5900#issuecomment-895513797
class _MISSING_TYPE(enum.Enum):
    MISSING = enum.auto()

MISSING: Final = _MISSING_TYPE.MISSING

if sys.version_info >= (3, 10):
    class KW_ONLY: ...

@overload
def asdict(obj: DataclassInstance) -> dict[str, Any]: ...
@overload
def asdict(obj: DataclassInstance, *, dict_factory: Callable[[list[tuple[str, Any]]], _T]) -> _T: ...
@overload
def astuple(obj: DataclassInstance) -> tuple[Any, ...]: ...
@overload
def astuple(obj: DataclassInstance, *, tuple_factory: Callable[[list[Any]], _T]) -> _T: ...

if sys.version_info >= (3, 11):
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type[_T]: ...
    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

elif sys.version_info >= (3, 10):
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> type[_T]: ...
    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

else:
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> type[_T]: ...
    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

# See https://github.com/python/mypy/issues/10750
@type_check_only
class _DefaultFactory(Protocol[_T_co]):
    def __call__(self) -> _T_co: ...

class Field(Generic[_T]):
    if sys.version_info >= (3, 14):
        __slots__ = (
            "name",
            "type",
            "default",
            "default_factory",
            "repr",
            "hash",
            "init",
            "compare",
            "metadata",
            "kw_only",
            "doc",
            "_field_type",
        )
    elif sys.version_info >= (3, 10):
        __slots__ = (
            "name",
            "type",
            "default",
            "default_factory",
            "repr",
            "hash",
            "init",
            "compare",
            "metadata",
            "kw_only",
            "_field_type",
        )
    else:
        __slots__ = ("name", "type", "default", "default_factory", "repr", "hash", "init", "compare", "metadata", "_field_type")
    name: str
    type: Type[_T] | str | Any
    default: _T | Literal[_MISSING_TYPE.MISSING]
    default_factory: _DefaultFactory[_T] | Literal[_MISSING_TYPE.MISSING]
    repr: bool
    hash: bool | None
    init: bool
    compare: bool
    metadata: types.MappingProxyType[Any, Any]

    if sys.version_info >= (3, 14):
        doc: str | None

    if sys.version_info >= (3, 10):
        kw_only: bool | Literal[_MISSING_TYPE.MISSING]

    if sys.version_info >= (3, 14):
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
            kw_only: bool,
            doc: str | None,
        ) -> None: ...
    elif sys.version_info >= (3, 10):
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
            kw_only: bool,
        ) -> None: ...
    else:
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
        ) -> None: ...

    def __set_name__(self, owner: Type[Any], name: str) -> None: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

# NOTE: Actual return type is 'Field[_T]', but we want to help type checkers
# to understand the magic that happens at runtime.
if sys.version_info >= (3, 14):
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> Any: ...

elif sys.version_info >= (3, 10):
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> Any: ...

else:
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> Any: ...

def fields(class_or_instance: DataclassInstance | type[DataclassInstance]) -> tuple[Field[Any], ...]: ...

# HACK: `obj: Never` typing matches if object argument is using `Any` type.
@overload
def is_dataclass(obj: Never) -> TypeIs[DataclassInstance | type[DataclassInstance]]: ...  # type: ignore[narrowed-type-not-subtype]  # pyright: ignore[reportGeneralTypeIssues]
@overload
def is_dataclass(obj: type) -> TypeIs[type[DataclassInstance]]: ...
@overload
def is_dataclass(obj: object) -> TypeIs[DataclassInstance | type[DataclassInstance]]: ...

class FrozenInstanceError(AttributeError): ...

class InitVar(Generic[_T]):
    __slots__ = ("type",)
    type: Type[_T]
    def __init__(self, type: Type[_T]) -> None: ...
    @overload
    def __class_getitem__(cls, type: Type[_T]) -> InitVar[_T]: ...  # pyright: ignore[reportInvalidTypeForm]
    @overload
    def __class_getitem__(cls, type: Any) -> InitVar[Any]: ...  # pyright: ignore[reportInvalidTypeForm]

if sys.version_info >= (3, 14):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
        module: str | None = None,
        decorator: _DataclassFactory = ...,
    ) -> type: ...

elif sys.version_info >= (3, 12):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
        module: str | None = None,
    ) -> type: ...

elif sys.version_info >= (3, 11):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type: ...

elif sys.version_info >= (3, 10):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> type: ...

else:
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> type: ...

def replace(obj: _DataclassT, /, **changes: Any) -> _DataclassT: ...



from _typeshed import Incomplete
from datetime import datetime
from typing import Any

from docker.types.daemon import CancellableStream

class DaemonApiMixin:
    def df(self) -> dict[str, Any]: ...
    def events(
        self,
        since: datetime | int | None = None,
        until: datetime | int | None = None,
        filters: dict[str, Any] | None = None,
        decode: bool | None = None,
    ) -> CancellableStream[Incomplete]: ...
    def info(self) -> dict[str, Any]: ...
    def login(
        self,
        username: str,
        password: str | None = None,
        email: str | None = None,
        registry: str | None = None,
        reauth: bool = False,
        dockercfg_path: str | None = None,
    ) -> dict[str, Any]: ...
    def ping(self) -> bool: ...
    def version(self, api_version: bool = True) -> dict[str, Any]: ...



def prepare_grant_uri(uri, client_id, response_type, redirect_uri=None, scope=None, state=None, **kwargs): ...
def prepare_token_request(grant_type, body: str = "", redirect_uri=None, **kwargs): ...
def parse_authorization_code_response(uri, state=None): ...
def parse_implicit_response(uri, state=None): ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Stats:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def active_users(self) -> int: ...
    async def active_users_async(self) -> int: ...
    def daily_stats(self, from_date: str | None = None, to_date: str | None = None) -> list[dict[str, Incomplete]]: ...
    async def daily_stats_async(
        self, from_date: str | None = None, to_date: str | None = None
    ) -> list[dict[str, Incomplete]]: ...



from _typeshed import GenericPath, StrOrBytesPath, StrPath
from collections.abc import Iterator
from typing import AnyStr

VCS_NAMES: list[str]
IGNORED_EXTENSIONS: list[str]

def list_directory(path: GenericPath[AnyStr], include_directories: bool = True) -> Iterator[AnyStr]: ...
def mtime(path: StrOrBytesPath) -> str: ...
def md5(path: StrPath) -> str: ...



from _typeshed import StrPath
from pathlib import Path
from re import Match, Pattern
from typing import ClassVar, Final

from docutils import nodes
from docutils.parsers.rst import Directive
from docutils.parsers.rst.states import SpecializedBody

__docformat__: Final = "reStructuredText"

def adapt_path(path: str, source: StrPath = "", root_prefix: StrPath = "") -> str: ...

class Include(Directive):
    standard_include_path: Path
    def read_file(self, path: StrPath) -> str: ...
    def as_literal_block(self, text: str) -> list[nodes.literal_block]: ...
    def as_code_block(self, text: str) -> list[nodes.literal_block]: ...
    def custom_parse(self, text: str) -> list[nodes.Node]: ...
    def insert_into_input_lines(self, text: str) -> None: ...

class Raw(Directive): ...
class Replace(Directive): ...

class Unicode(Directive):
    comment_pattern: Pattern[str]

class Class(Directive): ...

class Role(Directive):
    argument_pattern: Pattern[str]

class DefaultRole(Directive): ...
class Title(Directive): ...

class MetaBody(SpecializedBody):
    def field_marker(  # type: ignore[override]
        self, match: Match[str], context: list[str], next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def parsemeta(self, match: Match[str]): ...

class Meta(Directive):
    SMkwargs: ClassVar[dict[str, tuple[MetaBody]]]

class Date(Directive): ...
class TestDirective(Directive): ...



import socket
import threading
from _typeshed import FileDescriptorLike
from _typeshed.wsgi import WSGIEnvironment
from collections.abc import Callable
from selectors import SelectorKey, _EventMask
from ssl import SSLContext
from typing import Any, Protocol, type_check_only

from wsproto import ConnectionType, WSConnection
from wsproto.events import Request
from wsproto.frame_protocol import CloseReason

@type_check_only
class _ThreadClassProtocol(Protocol):
    name: str
    # this accepts any callable as the target, like `threading.Thread`
    def __init__(self, target: Callable[..., Any]) -> None: ...
    def start(self) -> None: ...

@type_check_only
class _EventClassProtocol(Protocol):
    def clear(self) -> None: ...
    def set(self) -> None: ...
    def wait(self, timeout: float | None = None) -> bool: ...

@type_check_only
class _SelectorClassProtocol(Protocol):
    # the signature of `register` here is the same as `selectors._BaseSelectorImpl` from the stdlib
    def register(self, fileobj: FileDescriptorLike, events: _EventMask, data: Any = None) -> SelectorKey: ...
    # the signature of `select` here is the same as `selectors.DefaultSelector` from the stdlib
    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, _EventMask]]: ...
    def close(self) -> None: ...

class Base:
    subprotocol: str | None
    sock: socket.socket | None
    receive_bytes: int
    ping_interval: float | None
    max_message_size: int | None
    pong_received: bool
    input_buffer: list[bytes | str]
    incoming_message: bytes | str | None
    incoming_message_len: int
    connected: bool
    is_server: bool
    close_reason: CloseReason
    close_message: str | None
    selector_class: type[_SelectorClassProtocol]
    event: _EventClassProtocol | threading.Event
    ws: WSConnection
    thread: _ThreadClassProtocol | threading.Thread
    def __init__(
        self,
        sock: socket.socket | None = None,
        connection_type: ConnectionType | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        thread_class: type[_ThreadClassProtocol] | None = None,
        event_class: type[_EventClassProtocol] | None = None,
        selector_class: type[_SelectorClassProtocol] | None = None,
    ) -> None: ...
    def handshake(self) -> None: ...
    # data can be antyhing. a special case is made for `bytes`, anything else is converted to `str`.
    def send(self, data: bytes | Any) -> None: ...
    def receive(self, timeout: float | None = None) -> bytes | str | None: ...
    def close(self, reason: CloseReason | None = None, message: str | None = None) -> None: ...
    def choose_subprotocol(self, request: Request) -> str | None: ...

class Server(Base):
    environ: WSGIEnvironment
    subprotocols: list[str]
    mode: str
    connected: bool
    def __init__(
        self,
        environ: WSGIEnvironment,
        subprotocols: list[str] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        thread_class: type[_ThreadClassProtocol] | None = None,
        event_class: type[_EventClassProtocol] | None = None,
        selector_class: type[_SelectorClassProtocol] | None = None,
    ) -> None: ...
    @classmethod
    def accept(
        cls,
        environ: WSGIEnvironment,
        subprotocols: list[str] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        thread_class: type[_ThreadClassProtocol] | None = None,
        event_class: type[_EventClassProtocol] | None = None,
        selector_class: type[_SelectorClassProtocol] | None = None,
    ) -> Server: ...
    def handshake(self) -> None: ...
    def choose_subprotocol(self, request: Request) -> str | None: ...

class Client(Base):
    host: str
    port: int
    path: str
    subprotocols: list[str]
    extra_headeers: list[tuple[bytes, bytes]]
    subprotocol: str | None
    connected: bool
    def __init__(
        self,
        url: str,
        subprotocols: list[str] | None = None,
        headers: dict[bytes, bytes] | list[tuple[bytes, bytes]] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        ssl_context: SSLContext | None = None,
        thread_class: type[_ThreadClassProtocol] | None = None,
        event_class: type[_EventClassProtocol] | None = None,
    ) -> None: ...
    @classmethod
    def connect(
        cls,
        url: str,
        subprotocols: list[str] | None = None,
        headers: dict[bytes, bytes] | list[tuple[bytes, bytes]] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        ssl_context: SSLContext | None = None,
        thread_class: type[_ThreadClassProtocol] | None = None,
        event_class: type[_EventClassProtocol] | None = None,
    ) -> Client: ...
    def handshake(self) -> None: ...
    def close(self, reason: CloseReason | None = None, message: str | None = None) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatchable

from ..classes.digraph import DiGraph

__all__ = [
    "prefix_tree",
    "prefix_tree_recursive",
    "random_labeled_tree",
    "random_labeled_rooted_tree",
    "random_labeled_rooted_forest",
    "random_unlabeled_tree",
    "random_unlabeled_rooted_tree",
    "random_unlabeled_rooted_forest",
]

@_dispatchable
def prefix_tree(paths) -> DiGraph[Incomplete]: ...
@_dispatchable
def prefix_tree_recursive(paths) -> DiGraph[Incomplete]: ...
@_dispatchable
def random_labeled_tree(n, *, seed=None): ...
@_dispatchable
def random_labeled_rooted_tree(n, *, seed=None): ...
@_dispatchable
def random_unlabeled_rooted_tree(n, *, number_of_trees=None, seed=None) -> Incomplete | list[Incomplete]: ...
@_dispatchable
def random_labeled_rooted_forest(n, *, seed=None): ...
@_dispatchable
def random_unlabeled_rooted_forest(n, *, q=None, number_of_forests=None, seed=None) -> Incomplete | list[Incomplete]: ...
@_dispatchable
def random_unlabeled_tree(n, *, number_of_trees=None, seed=None) -> Incomplete | list[Incomplete]: ...



from abc import abstractmethod
from typing import Any, NoReturn
from typing_extensions import disjoint_base

from gevent._types import _Loop
from greenlet import greenlet

class TrackedRawGreenlet(greenlet): ...

@disjoint_base
class SwitchOutGreenletWithLoop(TrackedRawGreenlet):
    @property
    @abstractmethod
    def loop(self) -> _Loop: ...
    @loop.setter
    def loop(self, value: _Loop) -> None: ...
    def switch(self) -> Any: ...
    def switch_out(self) -> NoReturn: ...

__all__ = ["TrackedRawGreenlet", "SwitchOutGreenletWithLoop"]



import sys
from _typeshed import FileDescriptorLike

def cancel_dump_traceback_later() -> None: ...
def disable() -> None: ...
def dump_traceback(file: FileDescriptorLike = ..., all_threads: bool = ...) -> None: ...

if sys.version_info >= (3, 14):
    def dump_c_stack(file: FileDescriptorLike = ...) -> None: ...

def dump_traceback_later(timeout: float, repeat: bool = ..., file: FileDescriptorLike = ..., exit: bool = ...) -> None: ...

if sys.version_info >= (3, 14):
    def enable(file: FileDescriptorLike = ..., all_threads: bool = ..., c_stack: bool = True) -> None: ...

else:
    def enable(file: FileDescriptorLike = ..., all_threads: bool = ...) -> None: ...

def is_enabled() -> bool: ...

if sys.platform != "win32":
    def register(signum: int, file: FileDescriptorLike = ..., all_threads: bool = ..., chain: bool = ...) -> None: ...
    def unregister(signum: int, /) -> None: ...



from . import icalendar as icalendar, vcard as vcard
from .base import VERSION as VERSION, Component, readComponents as readComponents, readOne as readOne

def iCalendar() -> Component: ...
def vCard() -> Component: ...



from PyInstaller.utils.win32.versioninfo import (
    FixedFileInfo,
    StringFileInfo,
    StringStruct,
    StringTable,
    VarFileInfo,
    VarStruct,
    VSVersionInfo,
)

# Everything below this line is the content from running `pyi-grab_version python3`
# ==============================================================================

# UTF-8
#
# For more details about fixed file info 'ffi' see:
# http://msdn.microsoft.com/en-us/library/ms646997.aspx
VSVersionInfo(
    ffi=FixedFileInfo(
        # filevers and prodvers should be always a tuple with four items: (1, 2, 3, 4)
        # Set not needed items to zero 0.
        filevers=(3, 13, 1150, 1013),
        prodvers=(3, 13, 1150, 1013),
        # Contains a bitmask that specifies the valid bits 'flags'r
        mask=0x3F,
        # Contains a bitmask that specifies the Boolean attributes of the file.
        flags=0x0,
        # The operating system for which this file was designed.
        # 0x4 - NT and there is no need to change it.
        OS=0x4,
        # The general type of file.
        # 0x1 - the file is an application.
        fileType=0x2,
        # The function of the file.
        # 0x0 - the function is not defined for this fileType
        subtype=0x0,
        # Creation date and time stamp.
        date=(0, 0),
    ),
    kids=[
        StringFileInfo(
            [
                StringTable(
                    "000004b0",
                    [
                        StringStruct("CompanyName", "Python Software Foundation"),
                        StringStruct("FileDescription", "Python Core"),
                        StringStruct("FileVersion", "3.13.1"),
                        StringStruct("InternalName", "Python DLL"),
                        StringStruct(
                            "LegalCopyright",
                            "Copyright © 2001-2024 Python Software Foundation. Copyright © 2000 BeOpen.com. Copyright © 1995-2001 CNRI. Copyright © 1991-1995 SMC.",
                        ),
                        StringStruct("OriginalFilename", "python3.dll"),
                        StringStruct("ProductName", "Python"),
                        StringStruct("ProductVersion", "3.13.1"),
                    ],
                )
            ]
        ),
        VarFileInfo([VarStruct("Translation", [0, 1200])]),
    ],
)



from dataclasses import dataclass

from numpy.typing import ArrayLike
from seaborn._stats.base import Stat

@dataclass
class Count(Stat): ...

@dataclass
class Hist(Stat):
    stat: str = "count"
    bins: str | int | ArrayLike = "auto"
    binwidth: float | None = None
    binrange: tuple[float, float] | None = None
    common_norm: bool | list[str] = True
    common_bins: bool | list[str] = True
    cumulative: bool = False
    discrete: bool = False
    def __post_init__(self) -> None: ...



import configparser
from logging import Logger
from typing import Any

from .manager import OptionManager

LOG: Logger

def load_config(config: str | None, extra: list[str], *, isolated: bool = False) -> tuple[configparser.RawConfigParser, str]: ...
def parse_config(option_manager: OptionManager, cfg: configparser.RawConfigParser, cfg_dir: str) -> dict[str, Any]: ...



import numpy as np
from numpy.typing import NDArray

from ._geometry import GeometryType
from ._typing import ArrayLike, ArrayLikeSeq, GeoArray, OptGeoArrayLikeSeq

def to_ragged_array(
    geometries: OptGeoArrayLikeSeq, include_z: bool | None = None, include_m: bool | None = None
) -> tuple[GeometryType, NDArray[np.float64], tuple[NDArray[np.int32], ...]]: ...
def from_ragged_array(
    geometry_type: GeometryType, coords: ArrayLike[float], offsets: ArrayLikeSeq[int] | None = None
) -> GeoArray: ...

__all__ = ["to_ragged_array", "from_ragged_array"]



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import Literal, overload

text_type = str

class Crypto:
    @staticmethod
    def sha1_hmac_hash(secret_key: str | ReadableBuffer, content: str | ReadableBuffer | None) -> str: ...
    @staticmethod
    def sha256_hmac_hash(secret_key: str | ReadableBuffer, content: str | ReadableBuffer | None) -> str: ...
    @overload
    @staticmethod
    def secure_compare(left: None, right: Iterable[str | bytes | bytearray]) -> Literal[False]: ...
    @overload
    @staticmethod
    def secure_compare(left: Iterable[str | bytes | bytearray], right: None) -> Literal[False]: ...
    @overload
    @staticmethod
    def secure_compare(left: None, right: None) -> Literal[False]: ...
    @overload
    @staticmethod
    def secure_compare(left: Iterable[str | bytes | bytearray], right: Iterable[str | bytes | bytearray]) -> bool: ...



from .rfc5849 import (
    SIGNATURE_HMAC_SHA1 as SIGNATURE_HMAC_SHA1,
    SIGNATURE_PLAINTEXT as SIGNATURE_PLAINTEXT,
    SIGNATURE_RSA_SHA1 as SIGNATURE_RSA_SHA1,
    SIGNATURE_TYPE_BODY as SIGNATURE_TYPE_BODY,
    SIGNATURE_TYPE_HEADER as SIGNATURE_TYPE_HEADER,
    SIGNATURE_TYPE_QUERY as SIGNATURE_TYPE_QUERY,
    AuthorizationServer as AuthorizationServer,
    ClientAuth as ClientAuth,
    ClientMixin as ClientMixin,
    OAuth1Request as OAuth1Request,
    ResourceProtector as ResourceProtector,
    TemporaryCredential as TemporaryCredential,
    TemporaryCredentialMixin as TemporaryCredentialMixin,
    TokenCredentialMixin as TokenCredentialMixin,
)

__all__ = [
    "OAuth1Request",
    "ClientAuth",
    "SIGNATURE_HMAC_SHA1",
    "SIGNATURE_RSA_SHA1",
    "SIGNATURE_PLAINTEXT",
    "SIGNATURE_TYPE_HEADER",
    "SIGNATURE_TYPE_QUERY",
    "SIGNATURE_TYPE_BODY",
    "ClientMixin",
    "TemporaryCredentialMixin",
    "TokenCredentialMixin",
    "TemporaryCredential",
    "AuthorizationServer",
    "ResourceProtector",
]



from typing import Final

mathbb: Final[dict[str, str]]
mathbf: Final[dict[str, str]]
mathbfit: Final[dict[str, str]]
mathcal: Final[dict[str, str]]
mathfrak: Final[dict[str, str]]
mathit: Final[dict[str, str]]
mathsf: Final[dict[str, str]]
mathsfbf: Final[dict[str, str]]
mathsfbfit: Final[dict[str, str]]
mathsfit: Final[dict[str, str]]
mathtt: Final[dict[str, str]]



from Xlib.xobject import (
    colormap as colormap,
    cursor as cursor,
    drawable as drawable,
    fontable as fontable,
    icccm as icccm,
    resource as resource,
)

__all__ = ["colormap", "cursor", "drawable", "fontable", "icccm", "resource"]



from re import Pattern
from typing import Final

EXCLUDE: Final[tuple[str, ...]]
IGNORE: Final[tuple[str, ...]]
MAX_LINE_LENGTH: Final = 79
INDENT_SIZE: Final = 4
WHITESPACE: Final[frozenset[str]]
STATISTIC_NAMES: Final[tuple[str, ...]]
NOQA_INLINE_REGEXP: Final[Pattern[str]]
NOQA_FILE: Final[Pattern[str]]
VALID_CODE_PREFIX: Final[Pattern[str]]



import email.message
import http.client
import re
from _ssl import _PasswordType
from _typeshed import Incomplete, MaybeNone, StrOrBytesPath
from collections.abc import Generator
from ssl import _SSLMethod
from typing import ClassVar, Final, Literal, TypeVar
from typing_extensions import Self

from .error import *

_T = TypeVar("_T", default=str)

__author__: Final[str]
__copyright__: Final[str]
__contributors__: Final[list[str]]
__license__: Final[str]
__version__: Final[str]

def has_timeout(timeout: float | None) -> bool: ...

debuglevel: Final[int]
RETRIES: Final[int]
DEFAULT_MAX_REDIRECTS: Final[int]
HOP_BY_HOP: Final[list[str]]
SAFE_METHODS: Final[tuple[str, ...]]
REDIRECT_CODES: Final[frozenset[int]]
CA_CERTS: Final[str]
DEFAULT_TLS_VERSION: Final[_SSLMethod]
URI: Final[re.Pattern[str]]

def parse_uri(uri: str) -> tuple[str | MaybeNone, str | MaybeNone, str | MaybeNone, str | MaybeNone, str | MaybeNone]: ...
def urlnorm(uri: str) -> tuple[str | MaybeNone, str | MaybeNone, str | MaybeNone, str | MaybeNone]: ...

re_url_scheme: Final[re.Pattern[str]]
re_unsafe: Final[re.Pattern[str]]

def safename(filename: str | bytes) -> str: ...

NORMALIZE_SPACE: Final[re.Pattern[str]]
USE_WWW_AUTH_STRICT_PARSING: Final[int]

class Authentication:
    path: Incomplete
    host: Incomplete
    credentials: Incomplete
    http: Incomplete
    def __init__(self, credentials, host, request_uri: str, headers, response, content, http) -> None: ...
    def depth(self, request_uri: str) -> int: ...
    def inscope(self, host: str, request_uri: str) -> bool: ...
    def request(self, method, request_uri, headers, content) -> None: ...
    def response(self, response, content) -> bool: ...
    def __eq__(self, auth: object) -> bool: ...
    def __ne__(self, auth: object) -> bool: ...
    def __lt__(self, auth: object) -> bool: ...
    def __gt__(self, auth: object) -> bool: ...
    def __le__(self, auth: object) -> bool: ...
    def __ge__(self, auth: object) -> bool: ...
    def __bool__(self) -> bool: ...

class BasicAuthentication(Authentication):
    def __init__(self, credentials, host, request_uri, headers, response, content, http) -> None: ...
    def request(self, method, request_uri, headers, content) -> None: ...

class DigestAuthentication(Authentication):
    challenge: Incomplete
    A1: Incomplete
    def __init__(self, credentials, host, request_uri, headers, response, content, http) -> None: ...
    def request(self, method, request_uri, headers, content, cnonce=None): ...
    def response(self, response, content) -> bool: ...

class HmacDigestAuthentication(Authentication):
    challenge: Incomplete
    hashmod: Incomplete
    pwhashmod: Incomplete
    key: Incomplete
    __author__: ClassVar[str]
    def __init__(self, credentials, host, request_uri, headers, response, content, http) -> None: ...
    def request(self, method, request_uri, headers, content) -> None: ...
    def response(self, response, content) -> bool: ...

class WsseAuthentication(Authentication):
    def __init__(self, credentials, host, request_uri, headers, response, content, http) -> None: ...
    def request(self, method, request_uri, headers, content) -> None: ...

class GoogleLoginAuthentication(Authentication):
    Auth: str
    def __init__(self, credentials, host, request_uri, headers, response, content, http) -> None: ...
    def request(self, method, request_uri, headers, content) -> None: ...

class FileCache:
    cache: Incomplete
    safe: Incomplete
    def __init__(self, cache, safe=...) -> None: ...
    def get(self, key): ...
    def set(self, key, value) -> None: ...
    def delete(self, key) -> None: ...

class Credentials:
    credentials: Incomplete
    def __init__(self) -> None: ...
    def add(self, name, password, domain: str = "") -> None: ...
    def clear(self) -> None: ...
    def iter(self, domain) -> Generator[tuple[str, str], None, None]: ...

class KeyCerts(Credentials):
    def add(self, key, cert, domain, password) -> None: ...  # type: ignore[override]
    def iter(self, domain) -> Generator[tuple[str, str, str], None, None]: ...  # type: ignore[override]

class AllHosts: ...

class ProxyInfo:
    bypass_hosts: Incomplete
    def __init__(
        self, proxy_type, proxy_host, proxy_port, proxy_rdns: bool = True, proxy_user=None, proxy_pass=None, proxy_headers=None
    ) -> None: ...
    def astuple(self): ...
    def isgood(self): ...
    def applies_to(self, hostname): ...
    def bypass_host(self, hostname): ...

def proxy_info_from_environment(method: Literal["http", "https"] = "http") -> ProxyInfo | None: ...
def proxy_info_from_url(url: str, method: Literal["http", "https"] = "http", noproxy: str | None = None) -> ProxyInfo: ...

class HTTPConnectionWithTimeout(http.client.HTTPConnection):
    proxy_info: Incomplete
    def __init__(self, host, port=None, timeout=None, proxy_info=None) -> None: ...
    sock: Incomplete
    def connect(self) -> None: ...

class HTTPSConnectionWithTimeout(http.client.HTTPSConnection):
    disable_ssl_certificate_validation: bool
    ca_certs: StrOrBytesPath | None
    proxy_info: Incomplete
    key_file: StrOrBytesPath | None
    cert_file: StrOrBytesPath | None
    key_password: _PasswordType | None
    def __init__(
        self,
        host: str,
        port: int | None = None,
        key_file: StrOrBytesPath | None = None,
        cert_file: StrOrBytesPath | None = None,
        timeout: float | None = None,
        proxy_info=None,
        ca_certs: StrOrBytesPath | None = None,
        disable_ssl_certificate_validation: bool = False,
        tls_maximum_version=None,
        tls_minimum_version=None,
        key_password: _PasswordType | None = None,
    ) -> None: ...
    sock: Incomplete
    def connect(self) -> None: ...

SCHEME_TO_CONNECTION: Final[dict[Literal["http", "https"], type[http.client.HTTPConnection]]]

class Http:
    proxy_info: Incomplete
    ca_certs: Incomplete
    disable_ssl_certificate_validation: bool
    tls_maximum_version: Incomplete
    tls_minimum_version: Incomplete
    connections: Incomplete
    cache: FileCache
    credentials: Credentials
    certificates: KeyCerts
    authorizations: list[Authentication]
    follow_redirects: bool
    redirect_codes: frozenset[int]
    optimistic_concurrency_methods: list[str]
    safe_methods: list[str]
    follow_all_redirects: bool
    ignore_etag: bool
    force_exception_to_status_code: bool
    timeout: float | None
    forward_authorization_headers: bool
    def __init__(
        self,
        cache: str | FileCache | None = None,
        timeout: float | None = None,
        proxy_info=...,
        ca_certs=None,
        disable_ssl_certificate_validation: bool = False,
        tls_maximum_version=None,
        tls_minimum_version=None,
    ) -> None: ...
    def close(self) -> None: ...
    def add_credentials(self, name, password, domain: str = "") -> None: ...
    def add_certificate(self, key, cert, domain, password=None) -> None: ...
    def clear_credentials(self) -> None: ...
    def request(self, uri, method: str = "GET", body=None, headers=None, redirections=5, connection_type=None): ...

class Response(dict[str, str | _T]):
    fromcache: bool
    version: int
    status: int
    reason: str
    previous: Response[_T] | None
    def __init__(self, info: http.client.HTTPResponse | email.message.Message | dict[str, _T]) -> None: ...
    @property
    def dict(self) -> Self: ...

__all__ = [
    "debuglevel",
    "FailedToDecompressContent",
    "Http",
    "HttpLib2Error",
    "ProxyInfo",
    "RedirectLimit",
    "RedirectMissingLocation",
    "Response",
    "RETRIES",
    "UnimplementedDigestAuthOptionError",
    "UnimplementedHmacDigestAuthOptionError",
]



from __future__ import annotations

from typing import cast
from typing_extensions import assert_type

import grpc.aio

# Interceptor casts
client_interceptors: list[grpc.aio.ClientInterceptor] = []
grpc.aio.insecure_channel("target", interceptors=client_interceptors)

server_interceptors: list[grpc.aio.ServerInterceptor] = []
grpc.aio.server(interceptors=server_interceptors)


# Metadata
async def metadata() -> None:
    metadata = await cast(grpc.aio.Call, None).initial_metadata()
    assert_type(metadata["foo"], grpc.aio._MetadataValue)
    for k in metadata:
        assert_type(k, str)

    for k, v in metadata.items():
        assert_type(k, str)
        assert_type(v, grpc.aio._MetadataValue)



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class DeviceCredentials:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get(
        self,
        user_id: str,
        client_id: str,
        type: str,
        fields: list[str] | None = None,
        include_fields: bool = True,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = False,
    ): ...
    async def get_async(
        self,
        user_id: str,
        client_id: str,
        type: str,
        fields: list[str] | None = None,
        include_fields: bool = True,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = False,
    ): ...
    def create(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def delete(self, id: str): ...
    async def delete_async(self, id: str): ...



from _typeshed import Incomplete

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "hamiltonian_path",
    "is_reachable",
    "is_strongly_connected",
    "is_tournament",
    "random_tournament",
    "score_sequence",
    "tournament_matrix",
]

@_dispatchable
def is_tournament(G: Graph[_Node]) -> bool: ...
@_dispatchable
def hamiltonian_path(G: Graph[_Node]) -> list[Incomplete]: ...
@_dispatchable
def random_tournament(n: int, seed: int | RandomState | None = None) -> DiGraph[Incomplete]: ...
@_dispatchable
def tournament_matrix(G: Graph[_Node]): ...
@_dispatchable
def score_sequence(G: Graph[_Node]) -> list[Incomplete]: ...
@_dispatchable
def is_reachable(G: Graph[_Node], s: _Node, t: _Node) -> bool: ...
@_dispatchable
def is_strongly_connected(G: Graph[_Node]) -> bool: ...



from collections.abc import Mapping
from typing import Any, Literal
from typing_extensions import TypeAlias

from ..extractor.common import _InfoDict
from ..utils._utils import NO_DEFAULT
from .common import FileDownloader

__all__ = ["FileDownloader", "get_suitable_downloader", "shorten_protocol_name"]

_Protocol: TypeAlias = Literal[
    "dash_frag_urls",
    "ftps",
    "http_dash_segments",
    "http_dash_segments_generator",
    "https",
    "m3u8",
    "m3u8_frag_urls",
    "m3u8_native",
    "rtmp_ffmpeg",
    "websocket_frag",
]

def get_suitable_downloader(
    info_dict: _InfoDict,
    params: Mapping[str, Any] = {},
    default: FileDownloader | type[NO_DEFAULT] = ...,
    protocol: _Protocol | None = None,
    to_stdout: bool = False,
) -> FileDownloader: ...
def shorten_protocol_name(proto: _Protocol, simplify: bool = False) -> str: ...



from .resource import Collection, Model

class Service(Model):
    id_attribute: str
    @property
    def name(self): ...
    @property
    def version(self): ...
    def remove(self): ...
    def tasks(self, filters=None): ...
    def update(self, **kwargs): ...
    def logs(self, **kwargs): ...
    def scale(self, replicas): ...
    def force_update(self): ...

class ServiceCollection(Collection[Service]):
    model: type[Service]
    def create(self, image, command=None, **kwargs): ...  # type:ignore[override]
    def get(self, service_id, insert_defaults=None): ...
    def list(self, **kwargs): ...

CONTAINER_SPEC_KWARGS: list[str]
TASK_TEMPLATE_KWARGS: list[str]
CREATE_SERVICE_KWARGS: list[str]
PLACEMENT_KWARGS: list[str]



from ._typing import SupportsRead, SupportsWrite
from .geometry.base import BaseGeometry
from .lib import Geometry

def loads(data: str) -> BaseGeometry: ...
def load(fp: SupportsRead[str]) -> BaseGeometry: ...
def dumps(ob: Geometry, trim: bool = False, rounding_precision: int = -1, **kw) -> str: ...
def dump(ob: Geometry, fp: SupportsWrite[str], *, trim: bool = False, rounding_precision: int = -1, **kw) -> None: ...



from _typeshed import Incomplete
from builtins import bool as _bool
from collections.abc import Generator, Mapping, Sequence
from contextlib import contextmanager
from types import TracebackType
from typing import overload
from typing_extensions import Self

import tensorflow as tf
from tensorflow import Tensor, UnconnectedGradients, Variable
from tensorflow._aliases import ContainerGradients, ContainerTensors, ContainerTensorsLike, Gradients, TensorLike

class ForwardAccumulator:
    def __init__(self, primals: Tensor, tangents: Tensor) -> None: ...
    def jvp(
        self, primals: Tensor, unconnected_gradients: tf.UnconnectedGradients = tf.UnconnectedGradients.NONE  # noqa: Y011
    ) -> Tensor | None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, typ: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None) -> None: ...

class GradientTape:
    def __init__(self, persistent: _bool = False, watch_accessed_variables: _bool = True) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, typ: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None) -> None: ...
    # Higher kinded types would be nice here and these overloads are a way to simulate some of them.
    @overload
    def gradient(
        self,
        target: ContainerTensors,
        sources: TensorLike,
        output_gradients: list[Tensor] | None = None,
        unconnected_gradients: UnconnectedGradients = ...,
    ) -> Gradients: ...
    @overload
    def gradient(
        self,
        target: ContainerTensors,
        sources: Sequence[Tensor],
        output_gradients: list[Tensor] | None = None,
        unconnected_gradients: UnconnectedGradients = ...,
    ) -> list[Gradients]: ...
    @overload
    def gradient(
        self,
        target: ContainerTensors,
        sources: Mapping[str, Tensor],
        output_gradients: list[Tensor] | None = None,
        unconnected_gradients: UnconnectedGradients = ...,
    ) -> dict[str, Gradients]: ...
    @overload
    def gradient(
        self,
        target: ContainerTensors,
        sources: ContainerTensors,
        output_gradients: list[Tensor] | None = None,
        unconnected_gradients: UnconnectedGradients = ...,
    ) -> ContainerGradients: ...
    @contextmanager
    def stop_recording(self) -> Generator[None]: ...
    def reset(self) -> None: ...
    def watch(self, tensor: ContainerTensorsLike) -> None: ...
    def watched_variables(self) -> tuple[Variable, ...]: ...
    def __getattr__(self, name: str) -> Incomplete: ...



import sys
from _typeshed import FileDescriptorLike
from collections.abc import Iterable
from select import error as error
from typing import Any

def select(
    rlist: Iterable[Any], wlist: Iterable[Any], xlist: Iterable[Any], timeout: float | None = None
) -> tuple[list[Any], list[Any], list[Any]]: ...

if sys.platform != "win32":
    __all__ = ["error", "poll", "select"]
else:
    __all__ = ["error", "select"]

class poll:
    def register(self, fd: FileDescriptorLike, eventmask: int = ...) -> None: ...
    def modify(self, fd: FileDescriptorLike, eventmask: int) -> None: ...
    def poll(self, timeout: float | None = None) -> list[tuple[int, int]]: ...
    def unregister(self, fd: FileDescriptorLike) -> None: ...



from __future__ import annotations

from wtforms import DateField, Field, Form, StringField
from wtforms.validators import Email, Optional

form = Form()
# on form we should accept any validator mapping
form.validate({"field": (Optional(),), "string_field": (Optional(), Email())})
form.validate({"field": [Optional()], "string_field": [Optional(), Email()]})

# both StringField validators and Field validators should be valid
# as inputs on a StringField
string_field = StringField(validators=(Optional(), Email()))
string_field.validate(form, (Optional(), Email()))

# but not on Field
field = Field(validators=(Optional(), Email()))  # type: ignore
field.validate(form, (Optional(), Email()))  # type: ignore

# unless we only pass the Field validator
Field(validators=(Optional(),))
field.validate(form, (Optional(),))

# DateField should accept Field validators but not StringField validators
date_field = DateField(validators=(Optional(), Email()))  # type: ignore
date_field.validate(form, (Optional(), Email()))  # type: ignore
DateField(validators=(Optional(),))

# for lists we can't be as strict so we won't get type errors here
Field(validators=[Optional(), Email()])
field.validate(form, [Optional(), Email()])
DateField(validators=[Optional(), Email()])
date_field.validate(form, [Optional(), Email()])



from collections.abc import Callable, Collection, Mapping
from typing import overload
from typing_extensions import TypeAlias, deprecated

from authlib.oauth2 import JsonRequest, OAuth2Error, OAuth2Request
from authlib.oauth2.rfc6749 import BaseGrant, ClientMixin
from authlib.oauth2.rfc6750 import BearerTokenGenerator

from .hooks import Hookable

_ServerResponse: TypeAlias = tuple[int, str, list[tuple[str, str]]]

class AuthorizationServer(Hookable):
    scopes_supported: Collection[str] | None
    def __init__(self, scopes_supported: Collection[str] | None = None) -> None: ...
    def query_client(self, client_id: str) -> ClientMixin: ...
    def save_token(self, token: dict[str, str | int], request: OAuth2Request) -> None: ...
    def generate_token(
        self,
        grant_type: str,
        client: ClientMixin,
        user=None,
        scope: str | None = None,
        expires_in: int | None = None,
        include_refresh_token: bool = True,
    ) -> dict[str, str | int]: ...
    def register_token_generator(self, grant_type: str, func: BearerTokenGenerator) -> None: ...
    def authenticate_client(self, request: OAuth2Request, methods: Collection[str], endpoint: str = "token") -> ClientMixin: ...
    def register_client_auth_method(self, method, func) -> None: ...
    def register_extension(self, extension) -> None: ...
    def get_error_uri(self, request, error) -> None: ...
    def send_signal(self, name, *args: object, **kwargs: object) -> None: ...
    def create_oauth2_request(self, request) -> OAuth2Request: ...
    def create_json_request(self, request) -> JsonRequest: ...
    def handle_response(self, status: int, body: Mapping[str, object], headers: Mapping[str, str]) -> object: ...
    def validate_requested_scope(self, scope: str) -> None: ...
    def register_grant(
        self, grant_cls: type[BaseGrant], extensions: Collection[Callable[[BaseGrant], None]] | None = None
    ) -> None: ...
    def register_endpoint(self, endpoint) -> None: ...
    def get_authorization_grant(self, request: OAuth2Request) -> BaseGrant: ...
    def get_consent_grant(self, request=None, end_user=None): ...
    def get_token_grant(self, request: OAuth2Request) -> BaseGrant: ...
    def create_endpoint_response(self, name, request=None): ...
    @overload
    @deprecated("The 'grant' parameter will become mandatory.")
    def create_authorization_response(self, request=None, grant_user=None) -> object: ...
    @overload
    def create_authorization_response(self, request=None, grant_user=None, grant=None) -> object: ...
    def create_token_response(self, request=None) -> _ServerResponse: ...
    def handle_error_response(self, request: OAuth2Request, error: OAuth2Error) -> object: ...



import datetime
from _typeshed import Incomplete, ReadableBuffer, SupportsItems
from collections.abc import Callable, Iterable, Mapping
from types import BuiltinFunctionType, FunctionType, ModuleType
from typing import Any, ClassVar, NoReturn, TypeVar
from typing_extensions import Self

from yaml.error import YAMLError as YAMLError
from yaml.nodes import MappingNode as MappingNode, Node as Node, ScalarNode as ScalarNode, SequenceNode as SequenceNode

_T = TypeVar("_T")

class RepresenterError(YAMLError): ...

class BaseRepresenter:
    yaml_representers: ClassVar[dict[type[Any], Callable[[BaseRepresenter, Any], Node]]]
    yaml_multi_representers: ClassVar[dict[type[Any], Callable[[BaseRepresenter, Any], Node]]]
    default_style: str | Incomplete
    sort_keys: bool
    default_flow_style: bool
    represented_objects: dict[int, Node]
    object_keeper: list[Any]
    alias_key: int | Incomplete
    def __init__(self, default_style: str | None = None, default_flow_style: bool = False, sort_keys: bool = True) -> None: ...
    def represent(self, data) -> None: ...
    def represent_data(self, data) -> Node: ...
    @classmethod
    def add_representer(cls, data_type: type[_T], representer: Callable[[Self, _T], Node]) -> None: ...
    @classmethod
    def add_multi_representer(cls, data_type: type[_T], representer: Callable[[Self, _T], Node]) -> None: ...
    def represent_scalar(self, tag: str, value, style: str | None = None) -> ScalarNode: ...
    def represent_sequence(self, tag: str, sequence: Iterable[Any], flow_style: bool | None = None) -> SequenceNode: ...
    def represent_mapping(
        self, tag: str, mapping: SupportsItems[Any, Any] | Iterable[tuple[Any, Any]], flow_style: bool | None = None
    ) -> MappingNode: ...
    def ignore_aliases(self, data) -> bool: ...

class SafeRepresenter(BaseRepresenter):
    inf_value: ClassVar[float]
    def ignore_aliases(self, data) -> bool: ...
    def represent_none(self, data) -> ScalarNode: ...
    def represent_str(self, data: str) -> ScalarNode: ...
    def represent_binary(self, data: ReadableBuffer) -> ScalarNode: ...
    def represent_bool(self, data: bool) -> ScalarNode: ...
    def represent_int(self, data: int) -> ScalarNode: ...
    def represent_float(self, data: float) -> ScalarNode: ...
    def represent_list(self, data: Iterable[Any]) -> SequenceNode: ...
    def represent_dict(self, data: SupportsItems[Any, Any] | Iterable[tuple[Any, Any]]) -> MappingNode: ...
    def represent_set(self, data: Iterable[Any]) -> MappingNode: ...
    def represent_date(self, data: datetime.date) -> ScalarNode: ...
    def represent_datetime(self, data: datetime.datetime) -> ScalarNode: ...
    def represent_yaml_object(self, tag: str, data, cls, flow_style: bool | None = None) -> MappingNode: ...
    def represent_undefined(self, data) -> NoReturn: ...

class Representer(SafeRepresenter):
    def represent_complex(self, data: complex) -> ScalarNode: ...
    def represent_tuple(self, data: Iterable[Any]) -> SequenceNode: ...
    def represent_name(self, data: BuiltinFunctionType | FunctionType) -> ScalarNode: ...
    def represent_module(self, data: ModuleType) -> ScalarNode: ...
    def represent_object(self, data) -> SequenceNode | MappingNode: ...
    def represent_ordered_dict(self, data: Mapping[Any, Any]) -> SequenceNode: ...

__all__ = ["BaseRepresenter", "SafeRepresenter", "Representer", "RepresenterError"]



import datetime

from ..core import WesternCalendar

class Paraguay(WesternCalendar):
    def get_heroes_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_founding_of_asuncion(self, year: int) -> tuple[datetime.date, str]: ...
    def get_boqueron_battle_victory_day(self, year: int) -> tuple[datetime.date, str]: ...



from logging import Logger
from typing import Any, TypeVar, overload

_K = TypeVar("_K")

log: Logger

@overload
def metadata_to_dict(obj: dict[_K, Any]) -> dict[_K, Any]: ...
@overload
def metadata_to_dict(obj: type) -> str: ...
@overload
def metadata_to_dict(obj: Any) -> Any: ...



from _typeshed import Incomplete

from reportlab.pdfbase.pdfdoc import PDFObject

__all__ = ("AcroForm",)

class PDFFromString(PDFObject):
    def __init__(self, s) -> None: ...
    def format(self, document): ...

class RadioGroup(PDFObject):
    TU: Incomplete
    Ff: Incomplete
    kids: Incomplete
    T: Incomplete
    V: Incomplete
    def __init__(self, name, tooltip: str = "", fieldFlags: str = "noToggleToOff required radio") -> None: ...
    def format(self, doc): ...

class AcroForm(PDFObject):
    formFontNames: Incomplete
    referenceMap: Incomplete
    fonts: Incomplete
    fields: Incomplete
    sigFlags: Incomplete
    extras: Incomplete
    def __init__(self, canv, **kwds) -> None: ...
    @property
    def canv(self): ...
    def fontRef(self, f): ...
    def format(self, doc): ...
    def colorTuple(self, c): ...
    def streamFillColor(self, c): ...
    def streamStrokeColor(self, c): ...
    def checkboxAP(
        self,
        key,
        value,
        buttonStyle: str = "circle",
        shape: str = "square",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        size: int = 20,
        dashLen: int = 3,
    ): ...
    @staticmethod
    def circleArcStream(size, r, arcs=(0, 1, 2, 3), rotated: bool = False): ...
    def zdMark(self, c, size, ds, iFontName): ...
    def getRef(self, obj): ...
    def getRefStr(self, obj): ...
    @staticmethod
    def stdColors(t, b, f): ...
    @staticmethod
    def varyColors(key, t, b, f): ...
    def checkForceBorder(
        self, x, y, width, height, forceBorder, shape, borderStyle, borderWidth, borderColor, fillColor
    ) -> None: ...
    def checkbox(
        self,
        checked: bool = False,
        buttonStyle: str = "check",
        shape: str = "square",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "required",
        forceBorder: bool = False,
        relative: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def radio(
        self,
        value=None,
        selected: bool = False,
        buttonStyle: str = "circle",
        shape: str = "circle",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "noToggleToOff required radio",
        forceBorder: bool = False,
        relative: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def makeStream(self, width, height, stream, **D): ...
    def txAP(
        self,
        key,
        value,
        iFontName,
        rFontName,
        fontSize,
        shape: str = "square",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        dashLen: int = 3,
        wkind: str = "textfield",
        labels=[],
        I=[],
        sel_bg: str = "0.600006 0.756866 0.854904 rg",
        sel_fg: str = "0 g",
    ): ...
    def makeFont(self, fontName): ...
    def textfield(
        self,
        value: str = "",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        relative: bool = False,
        maxlen: int = 100,
        fontName=None,
        fontSize=None,
        dashLen: int = 3,
    ): ...
    def listbox(
        self,
        value: str = "",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        relative: bool = False,
        fontName=None,
        fontSize=None,
        dashLen: int = 3,
        maxlen=None,
        options=[],
    ): ...
    def choice(
        self,
        value: str = "",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int = 1,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "combo",
        forceBorder: bool = False,
        relative: bool = False,
        fontName=None,
        fontSize=None,
        dashLen: int = 3,
        maxlen=None,
        options=[],
    ): ...
    def checkboxRelative(self, **kwds) -> None: ...
    def radioRelative(self, **kwds) -> None: ...
    def textfieldRelative(self, **kwds) -> None: ...
    def listboxRelative(self, **kwds) -> None: ...
    def choiceRelative(self, **kwds) -> None: ...
    @property
    def encRefStr(self): ...

class CBMark:
    opNames: Incomplete
    opCount: Incomplete
    ops: Incomplete
    points: Incomplete
    slack: Incomplete
    def __init__(self, ops, points, bounds, slack: float = 0.05) -> None: ...
    def scaledRender(self, size, ds: int = 0): ...



import sys
from _socket import _Address
from _typeshed import Unused
from platform import uname_result
from re import Pattern
from socket import socket
from typing import Final, Literal
from typing_extensions import TypeAlias

if sys.platform == "darwin":
    SUPPORTED_PROTOCOLS: Final[tuple[None, Literal["tcp"], Literal["unix"], Literal["darwin"]]]
    _Protocol: TypeAlias = Literal["tcp", "unix", "darwin"] | None
    DARWIN_DISPLAY_RE: Final[Pattern[str]]
else:
    SUPPORTED_PROTOCOLS: Final[tuple[None, Literal["tcp"], Literal["unix"]]]
    _Protocol: TypeAlias = Literal["tcp", "unix"] | None
uname: uname_result
DISPLAY_RE: Final[Pattern[str]]

def get_display(display: str | None) -> tuple[str, str | None, str | None, int, int]: ...
def get_socket(dname: _Address, protocol: _Protocol, host: _Address | None, dno: int) -> socket: ...
def new_get_auth(sock: socket, dname: Unused, protocol: _Protocol, host: Unused, dno: int) -> tuple[bytes, bytes]: ...
def old_get_auth(sock: Unused, dname: _Address, host: Unused, dno: Unused) -> tuple[str | Literal[b""], bytes]: ...

get_auth = new_get_auth



import functools
from _typeshed import Incomplete
from functools import total_ordering as total_ordering

NO_DEFAULT: Incomplete

def inspect_formatargspec(
    args,
    varargs=None,
    varkw=None,
    defaults=None,
    kwonlyargs=(),
    kwonlydefaults={},
    annotations={},
    formatarg=...,
    formatvarargs=...,
    formatvarkw=...,
    formatvalue=...,
    formatreturns=...,
    formatannotation=...,
): ...
def get_module_callables(mod, ignore=None): ...
def mro_items(type_obj): ...
def dir_dict(obj, raise_exc: bool = False): ...
def copy_function(orig, copy_dict: bool = True): ...
def partial_ordering(cls): ...

class InstancePartial(functools.partial[Incomplete]):
    def __get__(self, obj, obj_type): ...

class CachedInstancePartial(functools.partial[Incomplete]):
    __name__: Incomplete
    def __set_name__(self, obj_type, name) -> None: ...
    __doc__: Incomplete
    __module__: Incomplete
    def __get__(self, obj, obj_type): ...

partial = CachedInstancePartial

def format_invocation(name: str = "", args=(), kwargs=None, **kw): ...
def format_exp_repr(obj, pos_names, req_names=None, opt_names=None, opt_key=None): ...
def format_nonexp_repr(obj, req_names=None, opt_names=None, opt_key=None): ...
def wraps(func, injected=None, expected=None, **kw): ...
def update_wrapper(wrapper, func, injected=None, expected=None, build_from=None, **kw): ...

class FunctionBuilder:
    name: Incomplete
    def __init__(self, name, **kw) -> None: ...
    def get_sig_str(self, with_annotations: bool = True): ...
    def get_invocation_str(self): ...
    @classmethod
    def from_func(cls, func): ...
    def get_func(self, execdict=None, add_source: bool = True, with_dict: bool = True): ...
    def get_defaults_dict(self): ...
    def get_arg_names(self, only_required: bool = False): ...
    defaults: Incomplete
    def add_arg(self, arg_name, default=..., kwonly: bool = False) -> None: ...
    def remove_arg(self, arg_name) -> None: ...

class MissingArgument(ValueError): ...
class ExistingArgument(ValueError): ...

def noop(*args, **kwargs) -> None: ...



from _typeshed import Incomplete
from typing import Any, ClassVar, Literal, TypeVar, overload
from typing_extensions import Self, TypeAlias

from reportlab.lib.colors import Color

_AlignmentEnum: TypeAlias = Literal[0, 1, 2, 4]
# FIXME: There are some places in the code that expect upper-case versions
#        so I'm unsure whether those would work in stylesheets as well
_AlignmentStr: TypeAlias = Literal["left", "center", "centre", "right", "justify"]
_Alignment: TypeAlias = _AlignmentEnum | _AlignmentStr
_T = TypeVar("_T")

class PropertySet:
    defaults: ClassVar[dict[str, Any]]
    name: str
    parent: PropertySet | None
    def __init__(self, name: str, parent: PropertySet | None = None, **kw: Any) -> None: ...
    def refresh(self) -> None: ...
    def listAttrs(self, indent: str = "") -> None: ...
    def clone(self, name: str, parent: PropertySet | None = None, **kwds: Any) -> Self: ...
    # PropertySet can have arbitrary attributes
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...

class ParagraphStyle(PropertySet):
    # NOTE: We list the attributes this has for sure due to defaults
    fontName: str
    fontSize: float
    leading: float
    leftIndent: float
    rightIndent: float
    firstLineIndent: float
    alignment: _Alignment
    spaceBefore: float
    spaceAfter: float
    bulletFontName: str
    bulletFontSize: float
    bulletIndent: float
    textColor: Color
    backColor: Color | None
    wordWrap: Incomplete | None
    borderWidth: float
    borderPadding: float
    borderColor: Color | None
    borderRadius: float | None
    allowWidows: Incomplete
    allowOrphans: Incomplete
    textTransform: Incomplete | None
    endDots: Incomplete | None
    splitLongWords: Incomplete
    underlineWidth: float
    bulletAnchor: Literal["start", "middle", "end"] | float
    justifyLastLine: Incomplete
    justifyBreaks: Incomplete
    spaceShrinkage: float
    strikeWidth: float
    underlineOffset: float
    underlineGap: float
    strikeOffset: float
    strikeGap: float
    linkUnderline: Incomplete
    underlineColor: Color | None
    strikeColor: Color | None
    hyphenationLang: str
    embeddedHyphenation: Incomplete
    uriWasteReduce: float
    # NOTE: We redefine __init__ for the same reason
    def __init__(
        self,
        name: str,
        parent: PropertySet | None = None,
        *,
        fontName: str = ...,
        fontSize: float = ...,
        leading: float = ...,
        leftIndent: float = ...,
        rightIndent: float = ...,
        firstLineIndent: float = ...,
        alignment: _Alignment = ...,
        spaceBefore: float = ...,
        spaceAfter: float = ...,
        bulletFontName: str = ...,
        bulletFontSize: float = ...,
        bulletIndent: float = ...,
        textColor: Color = ...,
        backColor: Color | None = ...,
        wordWrap: Incomplete | None = ...,
        borderWidth: float = ...,
        borderPadding: float = ...,
        borderColor: Color | None = ...,
        borderRadius: float | None = ...,
        allowWidows=...,
        allowOrphans=...,
        textTransform: Incomplete | None = ...,
        endDots: Incomplete | None = ...,
        splitLongWords=...,
        underlineWidth: float = ...,
        bulletAnchor: Literal["start", "middle", "end"] | float = ...,
        justifyLastLine=...,
        justifyBreaks=...,
        spaceShrinkage: float = ...,
        strikeWidth: float = ...,
        underlineOffset: float = ...,
        underlineGap: float = ...,
        strikeOffset: float = ...,
        strikeGap: float = ...,
        linkUnderline=...,
        underlineColor: Color | None = ...,
        strikeColor: Color | None = ...,
        hyphenationLang: str = ...,
        embeddedHyphenation=...,
        uriWasteReduce: float = ...,
        **kw: Any,
    ) -> None: ...

def str2alignment(
    v: _AlignmentStr,
    __map__: dict[_AlignmentStr, _AlignmentEnum] = {"centre": 1, "center": 1, "left": 0, "right": 2, "justify": 4},
) -> _AlignmentEnum: ...

class LineStyle(PropertySet):
    # NOTE: We list the attributes this has for sure due to defaults
    width: float
    color: Color
    def prepareCanvas(self, canvas) -> None: ...
    # NOTE: We redefine __init__ for the same reason
    def __init__(
        self, name: str, parent: PropertySet | None = None, *, width: float = ..., color: Color = ..., **kw: Any
    ) -> None: ...

class ListStyle(PropertySet):
    # NOTE: We list the attributes this has for sure due to defaults
    leftIndent: float
    rightIndent: float
    bulletAlign: _Alignment
    bulletType: str
    bulletColor: Color
    bulletFontName: str
    bulletFontSize: float
    bulletOffsetY: float
    bulletDedent: Incomplete
    bulletDir: Incomplete
    bulletFormat: Incomplete | None
    start: Incomplete | None
    # NOTE: We redefine __init__ for the same reason
    def __init__(
        self,
        name: str,
        parent: PropertySet | None = None,
        *,
        leftIndent: float = ...,
        rightIndent: float = ...,
        bulletAlign: _Alignment = ...,
        bulletType: str = ...,
        bulletColor: Color = ...,
        bulletFontName: str = ...,
        bulletFontSize: float = ...,
        bulletOffsetY: float = ...,
        bulletDedent=...,
        bulletDir=...,
        bulletFormat: Incomplete | None = ...,
        start: Incomplete | None = ...,
        **kw: Any,
    ) -> None: ...

class StyleSheet1:
    byName: dict[str, PropertySet]
    byAlias: dict[str, PropertySet]
    def __init__(self) -> None: ...
    def __getitem__(self, key: str) -> PropertySet: ...
    @overload
    def get(self, key: str) -> PropertySet: ...
    @overload
    def get(self, key: str, default: _T) -> PropertySet | _T: ...
    def __contains__(self, key: str) -> bool: ...
    def has_key(self, key: str) -> bool: ...
    def add(self, style: PropertySet, alias: str | None = None) -> None: ...
    def list(self) -> None: ...

def getSampleStyleSheet() -> StyleSheet1: ...

__all__ = ("PropertySet", "ParagraphStyle", "str2alignment", "LineStyle", "ListStyle", "StyleSheet1", "getSampleStyleSheet")



ext_m3u: str
ext_x_targetduration: str
ext_x_media_sequence: str
ext_x_discontinuity_sequence: str
ext_x_program_date_time: str
ext_x_media: str
ext_x_playlist_type: str
ext_x_key: str
ext_x_stream_inf: str
ext_x_version: str
ext_x_allow_cache: str
ext_x_endlist: str
extinf: str
ext_i_frames_only: str
ext_x_asset: str
ext_x_bitrate: str
ext_x_byterange: str
ext_x_i_frame_stream_inf: str
ext_x_discontinuity: str
ext_x_cue_out: str
ext_x_cue_out_cont: str
ext_x_cue_in: str
ext_x_cue_span: str
ext_oatcls_scte35: str
ext_is_independent_segments: str
ext_x_map: str
ext_x_start: str
ext_x_server_control: str
ext_x_part_inf: str
ext_x_part: str
ext_x_rendition_report: str
ext_x_skip: str
ext_x_session_data: str
ext_x_session_key: str
ext_x_preload_hint: str
ext_x_daterange: str
ext_x_gap: str
ext_x_content_steering: str
ext_x_image_stream_inf: str
ext_x_images_only: str
ext_x_tiles: str



from enum import Enum

class Reservoir:
    def __init__(self) -> None: ...
    def borrow_or_take(self, now: int, can_borrow: bool | None) -> ReservoirDecision | None: ...
    def load_quota(self, quota: int | None, TTL: int | None, interval: int | None) -> None: ...
    @property
    def quota(self) -> int | None: ...
    @property
    def TTL(self) -> int | None: ...

class ReservoirDecision(Enum):
    TAKE = "take"
    BORROW = "borrow"
    NO = "no"



import re
from _typeshed import Incomplete

from . import base

spaceCharacters: str
SPACES_REGEX: re.Pattern[str]

class Filter(base.Filter[dict[str, Incomplete]]):
    spacePreserveElements: frozenset[str]

def collapse_spaces(text: str) -> str: ...



from collections.abc import Callable
from typing import Any, overload
from typing_extensions import Self, TypeAlias

from tensorflow import Tensor
from tensorflow._aliases import DTypeLike, ShapeLike, TensorCompatible

class Initializer:
    def __call__(self, shape: ShapeLike, dtype: DTypeLike | None = None) -> Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class Constant(Initializer):
    def __init__(self, value: TensorCompatible = 0.0) -> None: ...

class GlorotNormal(Initializer):
    def __init__(self, seed: int | None = None) -> None: ...

class GlorotUniform(Initializer):
    def __init__(self, seed: int | None = None) -> None: ...

class TruncatedNormal(Initializer):
    def __init__(self, mean: TensorCompatible = 0.0, stddev: TensorCompatible = 0.05, seed: int | None = None) -> None: ...

class RandomNormal(Initializer):
    def __init__(self, mean: TensorCompatible = 0.0, stddev: TensorCompatible = 0.05, seed: int | None = None) -> None: ...

class RandomUniform(Initializer):
    def __init__(self, minval: TensorCompatible = -0.05, maxval: TensorCompatible = 0.05, seed: int | None = None) -> None: ...

class Zeros(Initializer): ...

constant = Constant
glorot_normal = GlorotNormal
glorot_uniform = GlorotUniform
truncated_normal = TruncatedNormal
zeros = Zeros

_Initializer: TypeAlias = (  # noqa: Y047
    str | Initializer | type[Initializer] | Callable[[ShapeLike], Tensor] | dict[str, Any] | None
)

@overload
def get(identifier: None) -> None: ...
@overload
def get(identifier: str | Initializer | dict[str, Any] | type[Initializer]) -> Initializer: ...
@overload
def get(identifier: Callable[[ShapeLike], Tensor]) -> Callable[[ShapeLike], Tensor]: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from typing import ClassVar

class Environment:
    Development: ClassVar[Environment]
    QA: ClassVar[Environment]
    Sandbox: ClassVar[Environment]
    Production: ClassVar[Environment]
    All: ClassVar[dict[str, Environment]]
    __name__: str
    is_ssl: bool
    ssl_certificate: Incomplete
    def __init__(
        self,
        name,
        server: str,
        port,
        auth_url: str,
        is_ssl: bool,
        ssl_certificate,
        graphql_server: str = "",
        graphql_port: str = "",
    ) -> None: ...
    @property
    def base_url(self) -> str: ...
    @property
    def port(self) -> int: ...
    @property
    def auth_url(self) -> str: ...
    @property
    def protocol(self) -> str: ...
    @property
    def server(self) -> str: ...
    @property
    def server_and_port(self) -> str: ...
    @property
    def graphql_server(self) -> str: ...
    @property
    def graphql_port(self) -> str: ...
    @property
    def graphql_server_and_port(self) -> str: ...
    @staticmethod
    def parse_environment(environment: Environment | str | None) -> Environment | None: ...
    @staticmethod
    def braintree_root() -> str: ...



from .constants import (
    DEFAULT_LINUX_STORE as DEFAULT_LINUX_STORE,
    DEFAULT_OSX_STORE as DEFAULT_OSX_STORE,
    DEFAULT_WIN32_STORE as DEFAULT_WIN32_STORE,
    PROGRAM_PREFIX as PROGRAM_PREFIX,
)
from .errors import CredentialsNotFound as CredentialsNotFound, StoreError as StoreError
from .store import Store as Store



import html.parser as htmlparser
import re
from _frozen_importlib import ModuleSpec
from collections.abc import Sequence

from markdown import Markdown

spec: ModuleSpec
commentclose: re.Pattern[str]
commentabruptclose: re.Pattern[str]
blank_line_re: re.Pattern[str]

class HTMLExtractor(htmlparser.HTMLParser):
    empty_tags: set[str]
    lineno_start_cache: list[int]
    md: Markdown
    def __init__(self, md: Markdown, *args, **kwargs): ...
    inraw: bool
    intail: bool
    stack: list[str]
    cleandoc: list[str]
    @property
    def line_offset(self) -> int: ...
    def at_line_start(self) -> bool: ...
    def get_endtag_text(self, tag: str) -> str: ...
    def handle_starttag(self, tag: str, attrs: Sequence[tuple[str, str]]) -> None: ...  # type: ignore[override]
    def handle_empty_tag(self, data: str, is_block: bool) -> None: ...
    def handle_decl(self, data: str) -> None: ...
    def parse_bogus_comment(self, i: int, report: int = 0) -> int: ...
    def get_starttag_text(self) -> str: ...



from _typeshed import Incomplete

from authlib.oauth2 import OAuth2Error as OAuth2Error

__all__ = [
    "OAuth2Error",
    "InsecureTransportError",
    "InvalidRequestError",
    "InvalidClientError",
    "UnauthorizedClientError",
    "InvalidGrantError",
    "UnsupportedResponseTypeError",
    "UnsupportedGrantTypeError",
    "InvalidScopeError",
    "AccessDeniedError",
    "MissingAuthorizationError",
    "UnsupportedTokenTypeError",
    "MissingCodeException",
    "MissingTokenException",
    "MissingTokenTypeException",
    "MismatchingStateException",
]

class InsecureTransportError(OAuth2Error):
    error: str
    description: str
    @classmethod
    def check(cls, uri) -> None: ...

class InvalidRequestError(OAuth2Error):
    error: str

class InvalidClientError(OAuth2Error):
    error: str
    status_code: int
    def get_headers(self): ...

class InvalidGrantError(OAuth2Error):
    error: str

class UnauthorizedClientError(OAuth2Error):
    error: str

class UnsupportedResponseTypeError(OAuth2Error):
    error: str
    response_type: Incomplete
    def __init__(
        self,
        response_type,
        description=None,
        uri=None,
        status_code=None,
        state=None,
        redirect_uri=None,
        redirect_fragment: bool = False,
        error=None,
    ) -> None: ...
    def get_error_description(self): ...

class UnsupportedGrantTypeError(OAuth2Error):
    error: str
    grant_type: Incomplete
    def __init__(self, grant_type) -> None: ...
    def get_error_description(self): ...

class InvalidScopeError(OAuth2Error):
    error: str
    description: str

class AccessDeniedError(OAuth2Error):
    error: str
    description: str

class ForbiddenError(OAuth2Error):
    status_code: int
    auth_type: Incomplete
    realm: Incomplete
    def __init__(self, auth_type=None, realm=None) -> None: ...
    def get_headers(self): ...

class MissingAuthorizationError(ForbiddenError):
    error: str
    description: str

class UnsupportedTokenTypeError(ForbiddenError):
    error: str

class MissingCodeException(OAuth2Error):
    error: str
    description: str

class MissingTokenException(OAuth2Error):
    error: str
    description: str

class MissingTokenTypeException(OAuth2Error):
    error: str
    description: str

class MismatchingStateException(OAuth2Error):
    error: str
    description: str



from collections.abc import Iterable

from docutils.utils.math.mathml_elements import MathElement, mover, msub, msubsup, msup, mtd, munder, munderover

letters: dict[str, str]
ordinary: dict[str, str]
greek_capitals: dict[str, str]
functions: dict[str, str | None]
modulo_functions: dict[str, tuple[bool, bool, bool, str]]
math_alphabets: dict[str, str]
stretchables: dict[str, str]
operators: dict[str, str]
thick_operators: dict[str, str]
small_operators: dict[str, str]
movablelimits: tuple[str, ...]
spaces: dict[str, str]
accents: dict[str, str]
over: dict[str, tuple[str, float]]
under: dict[str, tuple[str, float]]
anomalous_chars: dict[str, str]
mathbb: dict[str, str]
matrices: dict[str, tuple[str, str]]
layout_styles: dict[str, dict[str, bool | int]]
fractions: dict[str, dict[str, bool | int | str] | dict[str, bool | int] | dict[str, int]]
delimiter_sizes: list[str]
bigdelimiters: dict[str, int]

def tex_cmdname(string: str) -> tuple[str, str]: ...
def tex_number(string: str) -> tuple[str, str]: ...
def tex_token(string: str) -> tuple[str, str]: ...
def tex_group(string: str) -> tuple[str, str]: ...
def tex_token_or_group(string: str) -> tuple[str, str]: ...
def tex_optarg(string: str) -> tuple[str, str]: ...
def parse_latex_math(root: MathElement, source: str) -> MathElement: ...
def handle_cmd(name: str, node: MathElement, string: str) -> tuple[MathElement, str]: ...
def handle_math_alphabet(name: str, node: MathElement, string: str) -> tuple[MathElement, str]: ...
def handle_script_or_limit(
    node: MathElement, c: str, limits: str = ""
) -> munderover | msubsup | munder | msub | mover | msup: ...
def begin_environment(node: MathElement, string: str) -> tuple[mtd, str]: ...
def end_environment(node: MathElement, string: str) -> tuple[MathElement, str]: ...
def tex_equation_columns(rows: Iterable[str]) -> int: ...
def align_attributes(rows: Iterable[str]) -> dict[str, str | bool]: ...
def tex2mathml(tex_math: str, as_block: bool = False) -> str: ...



from .base import CAP_STYLE as CAP_STYLE, JOIN_STYLE as JOIN_STYLE
from .collection import GeometryCollection as GeometryCollection
from .geo import box as box, mapping as mapping, shape as shape
from .linestring import LineString as LineString
from .multilinestring import MultiLineString as MultiLineString
from .multipoint import MultiPoint as MultiPoint
from .multipolygon import MultiPolygon as MultiPolygon
from .point import Point as Point
from .polygon import LinearRing as LinearRing, Polygon as Polygon

__all__ = [
    "box",
    "shape",
    "mapping",
    "Point",
    "LineString",
    "Polygon",
    "MultiPoint",
    "MultiLineString",
    "MultiPolygon",
    "GeometryCollection",
    "LinearRing",
    "CAP_STYLE",
    "JOIN_STYLE",
]



from _typeshed import Incomplete, ReadableBuffer
from typing import Final

DEBUG: Final[bool]
SCRAMBLE_LENGTH: Final[int]
sha1_new: Incomplete

def scramble_native_password(password: ReadableBuffer | None, message: ReadableBuffer | None) -> bytes: ...
def ed25519_password(password: ReadableBuffer, scramble: ReadableBuffer) -> bytes: ...
def sha2_rsa_encrypt(password: bytes, salt: bytes, public_key: bytes) -> bytes: ...
def sha256_password_auth(conn, pkt): ...
def scramble_caching_sha2(password: ReadableBuffer, nonce: ReadableBuffer) -> bytes: ...
def caching_sha2_password_auth(conn, pkt) -> Incomplete | None: ...



from collections.abc import Mapping

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["hits"]

@_dispatchable
def hits(
    G: Graph[_Node],
    max_iter: int | None = 100,
    tol: float | None = 1e-08,
    nstart: Mapping[_Node, float] | None = None,
    normalized: bool = True,
) -> tuple[dict[_Node, float], dict[_Node, float]]: ...



from typing import Final

PY3: Final[bool]

class DefusedXmlException(ValueError): ...

class DTDForbidden(DefusedXmlException):
    name: str
    sysid: str | None
    pubid: str | None
    def __init__(self, name: str, sysid: str | None, pubid: str | None) -> None: ...

class EntitiesForbidden(DefusedXmlException):
    name: str
    value: str | None
    base: str | None
    sysid: str | None
    pubid: str | None
    notation_name: str | None
    def __init__(
        self, name: str, value: str | None, base: str | None, sysid: str | None, pubid: str | None, notation_name: str | None
    ) -> None: ...

class ExternalReferenceForbidden(DefusedXmlException):
    context: str
    base: str | None
    sysid: str | None
    pubid: str | None
    def __init__(self, context: str, base: str | None, sysid: str | None, pubid: str | None) -> None: ...

class NotSupportedError(DefusedXmlException): ...



############################################################################
# Original work Copyright 2017 Palantir Technologies, Inc.                 #
# Original work licensed under the MIT License.                            #
# See ThirdPartyNotices.txt in the project root for license information.   #
# All modifications Copyright (c) Open Law Library. All rights reserved.   #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import logging
from typing import List, Optional, Union

from lsprotocol import types


log = logging.getLogger(__name__)


class PositionCodec:
    def __init__(
        self,
        encoding: Optional[
            Union[types.PositionEncodingKind, str]
        ] = types.PositionEncodingKind.Utf16,
    ):
        self.encoding = encoding

    @classmethod
    def is_char_beyond_multilingual_plane(cls, char: str) -> bool:
        return ord(char) > 0xFFFF

    def utf16_unit_offset(self, chars: str):
        """
        Calculate the number of characters which need two utf-16 code units.

        Arguments:
            chars (str): The string to count occurrences of utf-16 code units for.
        """
        return sum(self.is_char_beyond_multilingual_plane(ch) for ch in chars)

    def client_num_units(self, chars: str):
        """
        Calculate the length of `str` in client-supported UTF-[32|16|8] code units.

        Arguments:
            chars (str): The string to return the length in UTF-[32|16|8] code units for.
        """
        utf32_units = len(chars)
        if self.encoding == types.PositionEncodingKind.Utf32:
            return utf32_units

        if self.encoding == types.PositionEncodingKind.Utf8:
            return utf32_units + (self.utf16_unit_offset(chars) * 2)

        return utf32_units + self.utf16_unit_offset(chars)

    def position_from_client_units(
        self, lines: List[str], position: types.Position
    ) -> types.Position:
        """
        Convert the position.character from UTF-[32|16|8] code units to UTF-32.

        A python application can't use the character member of `Position`
        directly. As per specification it is represented as a zero-based line and
        character offset based on posible a UTF-[32|16|8] string representation.

        All characters whose code point exceeds the Basic Multilingual Plane are
        represented by 2 UTF-16 or 4 UTF-8 code units.

        The offset of the closing quotation mark in x="😋" is
        - 7 in UTF-8 representation
        - 5 in UTF-16 representation
        - 4 in UTF-32 representation

        see: https://github.com/microsoft/language-server-protocol/issues/376

        Arguments:
            lines (list):
                The content of the document which the position refers to.
            position (Position):
                The line and character offset in UTF-[32|16|8] code units.

        Returns:
            The position with `character` being converted to UTF-32 code units.
        """
        if len(lines) == 0:
            return types.Position(0, 0)
        if position.line >= len(lines):
            return types.Position(len(lines) - 1, self.client_num_units(lines[-1]))

        _line = lines[position.line]
        _line = _line.replace("\r\n", "\n")  # TODO: it's a bit of a hack
        _client_len = self.client_num_units(_line)
        _utf32_len = len(_line)

        if _client_len == 0:
            return types.Position(position.line, 0)

        _client_end_of_line = self.client_num_units(_line)
        if position.character > _client_end_of_line:
            position.character = _client_end_of_line - 1

        _client_index = 0
        utf32_index = 0
        while True:
            _is_searching_queried_position = _client_index < position.character
            _is_before_end_of_line = utf32_index < _utf32_len
            _is_searching_for_position = (
                _is_searching_queried_position and _is_before_end_of_line
            )
            if not _is_searching_for_position:
                break

            _current_char = _line[utf32_index]
            _is_double_width = PositionCodec.is_char_beyond_multilingual_plane(
                _current_char
            )
            if _is_double_width:
                if self.encoding == types.PositionEncodingKind.Utf32:
                    _client_index += 1
                if self.encoding == types.PositionEncodingKind.Utf8:
                    _client_index += 4
                _client_index += 2
            else:
                _client_index += 1
            utf32_index += 1

        position = types.Position(line=position.line, character=utf32_index)
        return position

    def position_to_client_units(
        self, lines: List[str], position: types.Position
    ) -> types.Position:
        """
        Convert the position.character from its internal UTF-32 representation
        to client-supported UTF-[32|16|8] code units.

        Arguments:
            lines (list):
                The content of the document which the position refers to.
            position (Position):
                The line and character offset in UTF-32 code units.

        Returns:
            The position with `character` being converted to UTF-[32|16|8] code units.
        """
        try:
            character = self.client_num_units(
                lines[position.line][: position.character]
            )
            return types.Position(
                line=position.line,
                character=character,
            )
        except IndexError:
            return types.Position(line=len(lines), character=0)

    def range_from_client_units(
        self, lines: List[str], range: types.Range
    ) -> types.Range:
        """
        Convert range.[start|end].character from UTF-[32|16|8] code units to UTF-32.

        Arguments:
            lines (list):
                The content of the document which the range refers to.
            range (Range):
                The line and character offset in UTF-[32|16|8] code units.

        Returns:
            The range with `character` offsets being converted to UTF-32 code units.
        """
        range_new = types.Range(
            start=self.position_from_client_units(lines, range.start),
            end=self.position_from_client_units(lines, range.end),
        )
        return range_new

    def range_to_client_units(
        self, lines: List[str], range: types.Range
    ) -> types.Range:
        """
        Convert range.[start|end].character from UTF-32 to UTF-[32|16|8] code units.

        Arguments:
            lines (list):
                The content of the document which the range refers to.
            range (Range):
                The line and character offset in  code units.

        Returns:
            The range with `character` offsets being converted to UTF-[32|16|8] code units.
        """
        return types.Range(
            start=self.position_to_client_units(lines, range.start),
            end=self.position_to_client_units(lines, range.end),
        )



import sys
from _typeshed import ReadableBuffer
from typing_extensions import TypeAlias, deprecated

# Many functions in binascii accept buffer objects
# or ASCII-only strings.
_AsciiBuffer: TypeAlias = str | ReadableBuffer

def a2b_uu(data: _AsciiBuffer, /) -> bytes: ...
def b2a_uu(data: ReadableBuffer, /, *, backtick: bool = False) -> bytes: ...

if sys.version_info >= (3, 11):
    def a2b_base64(data: _AsciiBuffer, /, *, strict_mode: bool = False) -> bytes: ...

else:
    def a2b_base64(data: _AsciiBuffer, /) -> bytes: ...

def b2a_base64(data: ReadableBuffer, /, *, newline: bool = True) -> bytes: ...
def a2b_qp(data: _AsciiBuffer, header: bool = False) -> bytes: ...
def b2a_qp(data: ReadableBuffer, quotetabs: bool = False, istext: bool = True, header: bool = False) -> bytes: ...

if sys.version_info < (3, 11):
    @deprecated("Deprecated since Python 3.9; removed in Python 3.11.")
    def a2b_hqx(data: _AsciiBuffer, /) -> bytes: ...
    @deprecated("Deprecated since Python 3.9; removed in Python 3.11.")
    def rledecode_hqx(data: ReadableBuffer, /) -> bytes: ...
    @deprecated("Deprecated since Python 3.9; removed in Python 3.11.")
    def rlecode_hqx(data: ReadableBuffer, /) -> bytes: ...
    @deprecated("Deprecated since Python 3.9; removed in Python 3.11.")
    def b2a_hqx(data: ReadableBuffer, /) -> bytes: ...

def crc_hqx(data: ReadableBuffer, crc: int, /) -> int: ...
def crc32(data: ReadableBuffer, crc: int = 0, /) -> int: ...
def b2a_hex(data: ReadableBuffer, sep: str | bytes = ..., bytes_per_sep: int = 1) -> bytes: ...
def hexlify(data: ReadableBuffer, sep: str | bytes = ..., bytes_per_sep: int = 1) -> bytes: ...
def a2b_hex(hexstr: _AsciiBuffer, /) -> bytes: ...
def unhexlify(hexstr: _AsciiBuffer, /) -> bytes: ...

class Error(ValueError): ...
class Incomplete(Exception): ...



from __future__ import annotations

from typing import Any

from setuptools.command.editable_wheel import EditableStrategy, _LinkTree, _StaticPth, _TopLevelFinder
from setuptools.config.expand import EnsurePackagesDiscovered
from setuptools.config.pyprojecttoml import _EnsurePackagesDiscovered

# We don't care about the __init__ methods, only about if an instance respects the Protocol
_: Any = object()

# Test EditableStrategy Protocol implementers
editable_strategy: EditableStrategy
editable_strategy = _StaticPth(_, _, _)
editable_strategy = _LinkTree(_, _, _, _)
editable_strategy = _TopLevelFinder(_, _)
# Not EditableStrategy due to incompatible __call__ method
editable_strategy = EnsurePackagesDiscovered(_)  # type: ignore
editable_strategy = _EnsurePackagesDiscovered(_, _, _)  # type: ignore



from collections.abc import Collection
from typing import NoReturn, overload
from typing_extensions import Self, TypeAlias

from .base import BaseGeometry
from .linestring import LineString, _ConvertibleToLineString
from .multilinestring import MultiLineString

__all__ = ["orient", "Polygon", "LinearRing"]

_ConvertibleToLinearRing: TypeAlias = _ConvertibleToLineString  # same alias but with better name for doc purposes
_PolygonShellLike: TypeAlias = Polygon | _ConvertibleToLinearRing | None
_PolygonHolesLike: TypeAlias = Collection[_ConvertibleToLinearRing | None] | None

class LinearRing(LineString):
    __slots__: list[str] = []
    def __new__(self, coordinates: _ConvertibleToLinearRing | None = None) -> Self: ...
    @property
    def is_ccw(self) -> bool: ...

class InteriorRingSequence:
    def __init__(self, parent: Polygon) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> LinearRing: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self, key: int) -> LinearRing: ...
    @overload
    def __getitem__(self, key: slice) -> list[LinearRing]: ...

class Polygon(BaseGeometry):
    __slots__: list[str] = []
    def __new__(self, shell: _PolygonShellLike = None, holes: _PolygonHolesLike = None) -> Self: ...
    @property
    def exterior(self) -> LinearRing: ...
    @property
    def interiors(self) -> list[LinearRing] | InteriorRingSequence: ...
    @property
    def coords(self) -> NoReturn: ...
    def svg(self, scale_factor: float = 1.0, fill_color: str | None = None, opacity: float | None = None) -> str: ...  # type: ignore[override]
    @classmethod
    def from_bounds(cls, xmin: float, ymin: float, xmax: float, ymax: float) -> Self: ...
    # more precise base overrides
    @property
    def boundary(self) -> MultiLineString: ...

def orient(polygon: Polygon, sign: float = 1.0) -> Polygon: ...



import datetime
import pathlib
from collections.abc import Callable
from typing import Final

TZAwareDate = datetime.date
YEAR_INTERVAL: Final = 30
TIME_ZONES: Final = ("America/Santiago", "Asia/Hong_Kong", "Asia/Taipei", "Asia/Tokyo")
pre_computed_equinoxes_path: Final[pathlib.Path]
pre_computed_solar_terms_path: Final[pathlib.Path]

def fromisoformat(iso: str) -> datetime.date: ...
def create_astronomical_data(progress: Callable[[int], int] | None = None) -> None: ...
def calculate_equinoxes(year: int, timezone: str = "UTC") -> tuple[TZAwareDate, TZAwareDate]: ...
def solar_term(year: int, degrees: int, timezone: str = "UTC") -> TZAwareDate: ...



from collections.abc import Generator

import gdb

def frame_for_id(id: int) -> gdb.Frame: ...
def select_frame(id: int) -> None: ...
def dap_frame_generator(frame_low: int, levels: int, include_all: bool) -> Generator[tuple[int, gdb.Frame]]: ...



from typing import Any

from ..extractor.common import _InfoDict
from .fragment import FragmentFD

class YoutubeLiveChatFD(FragmentFD):
    def real_download(self, filename: str, info_dict: _InfoDict) -> bool: ...
    @staticmethod
    def parse_live_timestamp(action: dict[str, Any]) -> int | None: ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable

from pyasn1.type.tag import TagSet

__all__ = ["decode"]

class AbstractScalarPayloadDecoder:
    def __call__(self, pyObject, asn1Spec, decodeFun: Unused = None, **options): ...

class BitStringPayloadDecoder(AbstractScalarPayloadDecoder):
    def __call__(self, pyObject, asn1Spec, decodeFun: Unused = None, **options): ...

class SequenceOrSetPayloadDecoder:
    def __call__(self, pyObject, asn1Spec, decodeFun: Callable[..., Incomplete] | None = None, **options): ...

class SequenceOfOrSetOfPayloadDecoder:
    def __call__(self, pyObject, asn1Spec, decodeFun: Callable[..., Incomplete] | None = None, **options): ...

class ChoicePayloadDecoder:
    def __call__(self, pyObject, asn1Spec, decodeFun: Callable[..., Incomplete] | None = None, **options): ...

TAG_MAP: dict[TagSet, AbstractScalarPayloadDecoder | SequenceOrSetPayloadDecoder | ChoicePayloadDecoder]
TYPE_MAP: dict[int, AbstractScalarPayloadDecoder | SequenceOrSetPayloadDecoder | ChoicePayloadDecoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemDecoder:
    TAG_MAP: dict[TagSet, AbstractScalarPayloadDecoder | SequenceOrSetPayloadDecoder | ChoicePayloadDecoder]
    TYPE_MAP: dict[int, AbstractScalarPayloadDecoder | SequenceOrSetPayloadDecoder | ChoicePayloadDecoder]

    def __init__(self, tagMap=..., typeMap=..., **ignored: Unused) -> None: ...
    def __call__(self, pyObject, asn1Spec, **options): ...

class Decoder:
    SINGLE_ITEM_DECODER: type[SingleItemDecoder]

    def __init__(self, *, tagMap=..., typeMap=..., **options: Unused) -> None: ...
    def __call__(self, pyObject, asn1Spec=None, **kwargs): ...

decode: Decoder



from collections.abc import Iterable, Sequence
from typing import Literal, TypeVar, overload
from typing_extensions import Self, TypeAlias

from matplotlib.colors import Colormap, LinearSegmentedColormap, ListedColormap
from matplotlib.typing import ColorType

__all__ = [
    "color_palette",
    "hls_palette",
    "husl_palette",
    "mpl_palette",
    "dark_palette",
    "light_palette",
    "diverging_palette",
    "blend_palette",
    "xkcd_palette",
    "crayon_palette",
    "cubehelix_palette",
    "set_color_codes",
]

_ColorT = TypeVar("_ColorT", bound=ColorType)

SEABORN_PALETTES: dict[str, list[str]]
MPL_QUAL_PALS: dict[str, int]
QUAL_PALETTE_SIZES: dict[str, int]
QUAL_PALETTES: list[str]

class _ColorPalette(list[_ColorT]):
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def as_hex(self) -> _ColorPalette[str]: ...

_RGBColorPalette: TypeAlias = _ColorPalette[tuple[float, float, float]]
_SeabornPaletteName: TypeAlias = Literal[
    "deep", "deep6", "muted", "muted6", "pastel", "pastel6", "bright", "bright6", "dark", "dark6", "colorblind", "colorblind6"
]

@overload
def color_palette(  # type: ignore[overload-overlap]
    palette: _SeabornPaletteName | None = None, n_colors: int | None = None, desat: float | None = None, *, as_cmap: Literal[True]
) -> list[str]: ...  # this might be a bug in seaborn because we expect the return type to be a Colormap instance
@overload
def color_palette(
    palette: str | Sequence[ColorType], n_colors: int | None = None, desat: float | None = None, *, as_cmap: Literal[True]
) -> Colormap: ...
@overload
def color_palette(
    palette: str | Sequence[ColorType] | None = None,
    n_colors: int | None = None,
    desat: float | None = None,
    as_cmap: Literal[False] = False,
) -> _RGBColorPalette: ...
@overload
def hls_palette(
    n_colors: int = 6, h: float = 0.01, l: float = 0.6, s: float = 0.65, *, as_cmap: Literal[True]
) -> ListedColormap: ...
@overload
def hls_palette(
    n_colors: int = 6, h: float = 0.01, l: float = 0.6, s: float = 0.65, as_cmap: Literal[False] = False
) -> _RGBColorPalette: ...
@overload
def husl_palette(
    n_colors: int = 6, h: float = 0.01, s: float = 0.9, l: float = 0.65, *, as_cmap: Literal[True]
) -> ListedColormap: ...
@overload
def husl_palette(
    n_colors: int = 6, h: float = 0.01, s: float = 0.9, l: float = 0.65, as_cmap: Literal[False] = False
) -> _RGBColorPalette: ...
@overload
def mpl_palette(name: str, n_colors: int = 6, *, as_cmap: Literal[True]) -> LinearSegmentedColormap: ...
@overload
def mpl_palette(name: str, n_colors: int = 6, as_cmap: Literal[False] = False) -> _RGBColorPalette: ...
@overload
def dark_palette(
    color: ColorType, n_colors: int = 6, reverse: bool = False, *, as_cmap: Literal[True], input: str = "rgb"
) -> LinearSegmentedColormap: ...
@overload
def dark_palette(
    color: ColorType, n_colors: int = 6, reverse: bool = False, as_cmap: Literal[False] = False, input: str = "rgb"
) -> _RGBColorPalette: ...
@overload
def light_palette(
    color: ColorType, n_colors: int = 6, reverse: bool = False, *, as_cmap: Literal[True], input: str = "rgb"
) -> LinearSegmentedColormap: ...
@overload
def light_palette(
    color: ColorType, n_colors: int = 6, reverse: bool = False, as_cmap: Literal[False] = False, input: str = "rgb"
) -> _RGBColorPalette: ...
@overload
def diverging_palette(
    h_neg: float,
    h_pos: float,
    s: float = 75,
    l: float = 50,
    sep: int = 1,
    n: int = 6,
    center: Literal["light", "dark"] = "light",
    *,
    as_cmap: Literal[True],
) -> LinearSegmentedColormap: ...
@overload
def diverging_palette(
    h_neg: float,
    h_pos: float,
    s: float = 75,
    l: float = 50,
    sep: int = 1,
    n: int = 6,
    center: Literal["light", "dark"] = "light",
    as_cmap: Literal[False] = False,
) -> _RGBColorPalette: ...
@overload
def blend_palette(
    colors: Iterable[ColorType], n_colors: int = 6, *, as_cmap: Literal[True], input: str = "rgb"
) -> LinearSegmentedColormap: ...
@overload
def blend_palette(
    colors: Iterable[ColorType], n_colors: int = 6, as_cmap: Literal[False] = False, input: str = "rgb"
) -> _RGBColorPalette: ...
def xkcd_palette(colors: Iterable[str]) -> _RGBColorPalette: ...
def crayon_palette(colors: Iterable[str]) -> _RGBColorPalette: ...
@overload
def cubehelix_palette(
    n_colors: int = 6,
    start: float = 0,
    rot: float = 0.4,
    gamma: float = 1.0,
    hue: float = 0.8,
    light: float = 0.85,
    dark: float = 0.15,
    reverse: bool = False,
    *,
    as_cmap: Literal[True],
) -> ListedColormap: ...
@overload
def cubehelix_palette(
    n_colors: int = 6,
    start: float = 0,
    rot: float = 0.4,
    gamma: float = 1.0,
    hue: float = 0.8,
    light: float = 0.85,
    dark: float = 0.15,
    reverse: bool = False,
    as_cmap: Literal[False] = False,
) -> _RGBColorPalette: ...
def set_color_codes(palette: str = "deep") -> None: ...
